/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include <list>
#include "DOMNodeCompare.hpp"
#include "XMLChCompare.hpp"
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMAttr.hpp>
#include <xercesc/dom/DOMDocument.hpp>

#include <assert.h>

using namespace std;

inline bool compareTwoAttributes(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *a, XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *b)
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *aowner = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *>(a)->getOwnerElement();
  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *bowner = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *>(b)->getOwnerElement();
  if(aowner != bowner) {
    return lessThan(aowner, bowner); // Attributes on different elements
  } else {
    return lessThan(a->getNodeName(), b->getNodeName()); // Attributes on the same element
  }
}

inline bool compareAttributeOther(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *a, XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *b)
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *owner = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *>(a)->getOwnerElement();
  if(owner == b) {
    return false; // Attributes come after their owner elements
  }
  return lessThan(owner, b);
}

inline bool compareOtherAttribute(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *a, XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *b)
{
  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *owner = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *>(b)->getOwnerElement();
  if(owner == a) {
    return true; // Owner elements come before their attributes
  }
  return lessThan(a, owner);
}

inline bool compare(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *a, XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *b)
{
  // Check they are on the same document
  if(a->getOwnerDocument() != b->getOwnerDocument()) {
    // Sort nodes from different trees in alphabetical order
    // according to the name of their document elements.
     return lessThan(a->getOwnerDocument()->getDocumentElement()->getNodeName(),
                     b->getOwnerDocument()->getDocumentElement()->getNodeName());
  }

  // Build a list of the ancestors of each node
  list<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> aparents;
  for(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *ia = a; ia != 0 && ia->getNodeType() != XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE;
      ia = ia->getParentNode()) {
    aparents.push_front(ia);
    if(ia->getParentNode() == 0) {
      // Nodes not attached to the tree are always greater than other nodes
      return false;
    }
  }
  list<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> bparents;
  for(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *ib = b; ib != 0 && ib->getNodeType() != XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE;
      ib = ib->getParentNode()) {
    bparents.push_front(ib);
    if(ib->getParentNode() == 0) {
      // Nodes not attached to the tree are always greater than other nodes
      return true;
    }
  }

  // find where the ancestor lists differ
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *parent = NULL;
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *abranch = NULL;
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *bbranch = NULL;

  if(aparents.size() < bparents.size()) {
    pair<list<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *>::iterator, list<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *>::iterator> common =
      mismatch(aparents.begin(), aparents.end(), bparents.begin());

    if(common.first == aparents.end()) {
      return true; // b is a decendant of a
    } else {
      //a is not directly related to b
      abranch = *common.first;
      bbranch = *common.second;
      parent = abranch->getParentNode();
      assert(parent == bbranch->getParentNode());
    }
  } else {
    pair<list<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *>::iterator, list<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *>::iterator> common =
      mismatch(bparents.begin(), bparents.end(), aparents.begin());

    if(common.first == bparents.end()) {
      return false; // a is a decendant of b
    } else if(common.first == (--bparents.end()) &&
       aparents.size() == bparents.size()) {
      //a is a sibling of b
      abranch = a;
      bbranch = b;
      parent = a->getParentNode();
      assert(parent == b->getParentNode());
    } else {
      //a is not directly related to b
      bbranch = *common.first;
      abranch = *common.second;
      parent = abranch->getParentNode();
      assert(parent == bbranch->getParentNode());
    }
  }

  // Which branch is the first child of the common parent?
  for(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *j = parent->getFirstChild(); j != 0; j = j->getNextSibling()) {
    if(abranch == j) {
      return true;
    }
    if(bbranch == j) {
      return false;
    }
  }
  // Something has gone wrong!!
  assert("Something wrong" == "in XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *compare");
  return false;
}

//Compare two DOM_Nodes for document order
bool lessThan(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *a, XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *b)
{
  if(a->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE) {
    if(b->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE) {
      return compareTwoAttributes(a, b);
    } else {
      return compareAttributeOther(a, b);
    }
  } else {
    if(b->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE) {
      return compareOtherAttribute(a, b);
    } else {
      return compare(a, b);
    }
  }
}
