/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "v2NSUtils.hpp"
#include <xercesc/dom/DOMAttr.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/XMLUniDefs.hpp>
#include "../context/Context.hpp"

XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *v2NSUtils::returnOwnerElement(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
  if(node!=NULL) {
    //this is to ensure that we have an element node
    XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *parent = NULL;
    if(node->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE) {
      parent = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *>(node)->getOwnerElement();
    }
    else if(node->getNodeType() != XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE) {
      parent = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *>(node->getParentNode());
    }
    else {
      parent = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *>(node);
    }

    return parent;
  } else {
    return NULL;
  }
}

const XMLCh *v2NSUtils::getLocalName(const XMLCh *name)
{
  unsigned int i = 0;
  while(name[i]!=XERCES_CPP_NAMESPACE_QUALIFIER chNull && name[i]!=XERCES_CPP_NAMESPACE_QUALIFIER chColon)
  {
    i++;
  }

  if(name[i]==XERCES_CPP_NAMESPACE_QUALIFIER chColon)
  {
    return name+i+1;
  }
  else
  {
    return name;
  }
}

const XMLCh *v2NSUtils::getLocalName(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
  const XMLCh *result = node->getLocalName();

  if(result==NULL || result[0]==0 ) {
    result = node->getNodeName();
  }
  return result;
}


const XMLCh *v2NSUtils::getPrefix(const XMLCh *name)
{
  unsigned int i = 0;
  while(name[i]!=XERCES_CPP_NAMESPACE_QUALIFIER chNull && name[i]!=XERCES_CPP_NAMESPACE_QUALIFIER chColon)
  {
    ++i;
  }

  XMLCh *result = NULL;
  if(name[i]==XERCES_CPP_NAMESPACE_QUALIFIER chColon)
  {
    result = new XMLCh[i+1];
    XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(result, name, 0, i);
  }
  return result;
}

const XMLCh *v2NSUtils::getPrefix(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node,
                                  const Context *context)
{
  const XMLCh *result = node->getPrefix();

  if(result==NULL || result[0]==0) {
    result = context->getInheritedNSPrefix(node->getNamespaceURI());
  }

  return result;
}

XMLCh *v2NSUtils::qualifyName(const XMLCh *prefix, const XMLCh *name)
{
  XMLCh *result = NULL;
  if (prefix && prefix[0]) {
    // result = prefix + ":" + name
    unsigned int prefixLen = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(prefix);
    unsigned int nameLen = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(name);
    result = new XMLCh[prefixLen+nameLen+2];

    XERCES_CPP_NAMESPACE_QUALIFIER XMLString::moveChars(result,prefix,prefixLen);
    result[prefixLen]=XERCES_CPP_NAMESPACE_QUALIFIER chColon;
    XERCES_CPP_NAMESPACE_QUALIFIER XMLString::copyString(result+prefixLen+1,name);
  } else {
    result = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(name);
  }

  return result;
}

XMLCh *v2NSUtils::qualifyName(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node,
                                    const Context *context)
{
  return qualifyName(getPrefix(node, context), getLocalName(node));
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *v2NSUtils::getParent(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
  if(node->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE) {
    return (static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *>(node))->getOwnerElement();
  }
  else {
    return node->getParentNode();
  }
}

XMLCh *
v2NSUtils::getExpandedName(const XMLCh *namespaceURI,
                           const XMLCh *name)
{
  // result = namespaceURI + "<" + name
  unsigned int uriLen = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(namespaceURI);
  unsigned int nameLen = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(name);
  XMLCh *result = new XMLCh[uriLen+nameLen+2];

  XERCES_CPP_NAMESPACE_QUALIFIER XMLString::moveChars(result,namespaceURI,uriLen);
  result[uriLen]=XERCES_CPP_NAMESPACE_QUALIFIER chOpenAngle;
  XERCES_CPP_NAMESPACE_QUALIFIER XMLString::copyString(result+uriLen+1,name);
  return result;
}

static void addBindings(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *AttrNode, namespaceMapType *namespaceMap);

namespaceMapType *v2NSUtils::createNamespaceMap(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *contextNode)
{
	namespaceMapType *namespaceMap = new namespaceMapType;

	//If this isn't an element, then return nothing
	if(contextNode==NULL || contextNode->getNodeType() != XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE) {
		return namespaceMap;
	}//if

	//We always add the default namespace
	(*namespaceMap)[XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("xml")]
           = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("http://www.w3.org/XML/1998/namespace");

	XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *tmpNode = contextNode;
	while(tmpNode!=NULL) {
		//For each element, go through the attributes, and add any namespace bindings..
		addBindings(tmpNode, namespaceMap);

		tmpNode = tmpNode->getParentNode();
	}//while

  return namespaceMap;

}//createNamespaceMap


static void addBindings(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *AttrNode, namespaceMapType *namespaceMap)
{
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNamedNodeMap *attrs = AttrNode->getAttributes();

  if (attrs != 0)
  {
    for (unsigned int cnt = 0; cnt < attrs->getLength(); cnt++)
    {
			XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *tmpAttr = attrs->item(cnt);

			//Check to see if this attribute starts with xmlns
      //If the name doesn't start with xmlns: (and its not xmlns) then skip it
			const XMLCh *attrName = tmpAttr->getNodeName();
      if(attrName[0]==XERCES_CPP_NAMESPACE_QUALIFIER chLatin_x
         && attrName[1]==XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m
         && attrName[2]==XERCES_CPP_NAMESPACE_QUALIFIER chLatin_l
         && attrName[3]==XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n
         && attrName[4]==XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s
         && (attrName[5]==XERCES_CPP_NAMESPACE_QUALIFIER chNull || attrName[5]==XERCES_CPP_NAMESPACE_QUALIFIER chColon))
      {
			//Get uri
			const XMLCh *uri = tmpAttr->getNodeValue();

			//Figure out prefix
        const XMLCh *prefix = NULL;

        //A prefix was given
        if(attrName[5]==XERCES_CPP_NAMESPACE_QUALIFIER chColon)
        {
          prefix = attrName+6;
        }

			//If the prefix wasn't empty, then the uri can't be empty
			//In other words, consider the case of "" being a valid uri for a default binding
        if((prefix==NULL || prefix[0]==0) || (uri!=NULL && uri[0]!=0))
        {

			//Add namespace, if not already there / shadowed
			namespaceMapType::iterator tmpIter = namespaceMap->find(prefix);
          if(tmpIter == namespaceMap->end())
                        {
            (*namespaceMap)[XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(prefix)] = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(uri);
                        }
        }//if
      }//if
		}//for
	}//if (attrs != 0)
}//addBindings
