/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _V2UTILS_HPP
#define _V2UTILS_HPP

#include <config.h>

#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/util/XMLString.hpp>
#include <string>

#ifdef TEST
#include "../testharness/Test.h"
#include "../testharness/TestSuite.h"
#include "../testharness/TestCase.h"
#include "../testharness/TestCaller.h"
#endif

#include "../defs/XMLScriptTypeDefs.hpp"

class DeadNodeList;
using namespace std;

/**class with methods that are used by v2 that are DOM related*/
class PATHAN_EXPORT v2Utils
#ifdef TEST
  : public TestCase
#endif

{
public:

  //If ascii output is set, then serialising nodes won't escape special characters and such
  static void setAsciiOutput(bool doAO);
  static bool getDoAsciiOuput(void);

  ///returns a c++ string from a DOMString
  static string DSLtranscode(const XMLCh *DOMstr);
  ///returns node type as a string. caller does not own string
  static const XMLCh *getNodeTypeAsString(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);
  ///returns the recursive text content of a node. caller owns string
  static XMLCh *getStringFromNode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);
  ///returns the recursive text content of a node converted to a double
  static XSFloat getDoubleFromNode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);

  /** Check if text content of any of my nodes is the same
   as any of theirs. Naieve method, O(n^2), multiple calls
   to getStringFromNode() for the otherNodes vector which
   could (should) be cached.*/
  static bool compareNodeLists(const DeadNodeList *myNodes,
			       const DeadNodeList *otherNodes);
  /** Comparison of two lists. O(n) */
  static bool compareNodeListsLessThan(const DeadNodeList *myNodes,
				       const DeadNodeList *otherNodes);
  static bool compareNodeListsGreaterThan(const DeadNodeList *myNodes,
					  const DeadNodeList *otherNodes);

  /** Obtain index of node with respect to it's siblings of same name */
  static int indexNode(const DeadNodeList *nodeList);
  static int indexNode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *indexNode);

  /// get the number of siblings of candidate with the same name
  static int getMaxNode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *candidate);

  /// print the node to a DOMString. caller owns string
  static XMLCh *printNodeToString(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);

  ///checks to see if child is a desendant of parent;
  static bool isAncestor(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *child);

  ///Convert a 32 bit Unicode character into ASCII-7 and entity references
  static string UnicodeToEntity (unsigned int inUnicode);

  ///Convert a 32 bit Unicode character into UTF-8
  static string UnicodeToUTF8 (unsigned int inUnicode);

  ///Convert a string into a UTF-8 encoded version
  static string StringToUTF8(const XMLCh *instr);

  ///Return the actual Unicode character index of the byte pointed to by 'offset'. -1 is returned in case of error.
  static int UnicodeIndexOfUTF8(const char *origString, const char *offset);

  //Get pointer into resultUTF's buffer, starting at Unicode character matchStartIndex
  static string UTF8StringAtUnicodeIndex(const char *origString, unsigned int index);

  ///Escape all attributes and text nodes that are attached to the node given.
  static void escapeNode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node);

  ///Escape a string. Boolean says escape quotes. caller owns string
  static XMLCh *escapeString(const XMLCh *str, bool escapeQuot);

  // Stream node & children. caller owns string
  static XMLCh *serialiseNode(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *toWrite);

  /** returns true if n is NaN */
  static bool isNaN(XSFloat n);

  /** returns true if n is Infinite */
  static bool isInfinite(XSFloat n);

  /** returns true if n is negative*/
  static bool isNegative(XSFloat n);

#ifdef TEST
  /// constructor for testing
  v2Utils(string name);
  ///Test setup
  void setUp(void);
  ///Test body
  static Test *suite(void);
  void testGetStringFromNode(void);
  void testCompareNodeLists(void);
  void testCompareNodeListsLessThan(void);
  void testCompareNodeListsGreaterThan(void);
#endif
private:
  static bool asciiOutput;
};
#endif
