/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "Context.hpp"
#include "../simpleVariables/VariableStore.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../DOMutils/v2NSUtils.hpp"
#include "../exceptions/ContextException.hpp"
#include "../functionAPI/FunctionLookup.hpp"
#include <xercesc/util/XMLUniDefs.hpp>

#include <assert.h>

// ""
static const XMLCh textEmpty[] =
{
   XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "http://www.w3.org/2000/xmlns/"
static const XMLCh textHttpXmlns[] =
{
   XERCES_CPP_NAMESPACE_QUALIFIER chLatin_h, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_p, XERCES_CPP_NAMESPACE_QUALIFIER chColon,
   XERCES_CPP_NAMESPACE_QUALIFIER chForwardSlash, XERCES_CPP_NAMESPACE_QUALIFIER chForwardSlash, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_w, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_w, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_w, XERCES_CPP_NAMESPACE_QUALIFIER chPeriod,
   XERCES_CPP_NAMESPACE_QUALIFIER chLatin_w, XERCES_CPP_NAMESPACE_QUALIFIER chDigit_3, XERCES_CPP_NAMESPACE_QUALIFIER chPeriod,
   XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_g, XERCES_CPP_NAMESPACE_QUALIFIER chForwardSlash,
   XERCES_CPP_NAMESPACE_QUALIFIER chDigit_2, XERCES_CPP_NAMESPACE_QUALIFIER chDigit_0, XERCES_CPP_NAMESPACE_QUALIFIER chDigit_0, XERCES_CPP_NAMESPACE_QUALIFIER chDigit_0, XERCES_CPP_NAMESPACE_QUALIFIER chForwardSlash,
   XERCES_CPP_NAMESPACE_QUALIFIER chLatin_x, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_l, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chForwardSlash,
   XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "xmlns"
static const XMLCh textXmlns[] =
{
   XERCES_CPP_NAMESPACE_QUALIFIER chLatin_x, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_l, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s,
   XERCES_CPP_NAMESPACE_QUALIFIER chNull
};


Context::Context(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *current,
                 XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *trigger,
                 VariableStore *varStore,
                 const FunctionLookup *funcTable,
                 XPathResult::resultType resultType)
  :  _currentContext(current), _triggerContext(trigger),
     _varStore(varStore), _functionTable(funcTable),
	 _resultType(resultType)
{
  _contextSize = 1;
  _contextPosition = 1;
  _creativeMode = false;
  _type = Context::BASE_CONTEXT;
}

Context::Context()
  : _currentContext(NULL), _triggerContext(NULL),
    _varStore(NULL), _functionTable(NULL)
{
  _contextSize = 0;
  _contextPosition = 0;
  _creativeMode = false;
  _type = Context::BASE_CONTEXT;
}

Context::~Context()
{
}

Context::Context(const Context &other)
  : _creativeMode(false), _currentContext(other._currentContext),
    _triggerContext(other._triggerContext),
    _contextSize(other._contextSize),
    _contextPosition(other._contextPosition),
    _type(other._type), _varStore(other._varStore),
    _functionTable(other._functionTable),
	_resultType(other._resultType)
{
}

Context &Context::operator=(const Context &other)
{
  _currentContext = other._currentContext;
  _triggerContext = other._triggerContext;

  _contextSize = other._contextSize;
  _contextPosition = other._contextPosition;

  _type = other._type;
  _creativeMode = false;
  _resultType= other._resultType;

  return *this;
}

//Context *Context::clone() const
//{
//  return new Context(*this);
//}

void Context::changeTrigger(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
  _triggerContext = node;
}

void Context::changeCurrent(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
  _currentContext = node;
}

void Context::changeCurrentInLoop(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
  _currentContext = node;
}

void Context::prepareForLoop()
{
  //Nothing to do
}

const FunctionLookup *Context::getFunctionTable() const
{
  return _functionTable;
}

void Context::setContextSize(unsigned int size)
{
  _contextSize = size;
}

void Context::setContextPosition(unsigned int pos)
{
  _contextPosition = pos;
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *Context::getTriggerContext() const
{
  return _triggerContext;
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *Context::getCurrentContext() const
{
  return _currentContext;
}

const VariableStore *Context::getVariableStore() const
{
  return _varStore;
}

unsigned int Context::getContextSize() const
{
  return _contextSize;
}

unsigned int Context::getContextPosition() const
{
  return _contextPosition;
}

Context::ContextType Context::getContextType() const
{
  return _type;
}

const XMLCh *Context::getDefaultExprNS() const
{
  return textEmpty;
}

XMLCh *Context::getExpandedName(const XMLCh *qname) const
{
  const XMLCh *expandedName = NULL;
  const XMLCh *prefix = v2NSUtils::getPrefix(qname);

  if(prefix == NULL) {
    //lets see if we put it in the null ns or if exprns is defined
    return v2NSUtils::getExpandedName(getDefaultExprNS(), qname);
  }
  else {
    //lets get what the prefix is bound to

    const XMLCh *ns = getInheritedPrefixNS(prefix);

    if(ns==NULL || ns[0]==0) {
      throw ContextException("The prefix specified is not bound to a URI");
    }

    return v2NSUtils::getExpandedName(ns, v2NSUtils::getLocalName(qname));
  }
}

const XMLCh *Context::getInheritedPrefixNS(const XMLCh *prefix) const
{
  //possible optimization - create a hash of prefix to URI once rather
  //then lookup every time

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *parent = v2NSUtils::returnOwnerElement(_triggerContext);

  while(parent!=NULL && parent->getNodeType() != XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE) {
    const XMLCh *uri = parent->getAttributeNS(textHttpXmlns, prefix);

    if (uri != 0 && *uri != 0) {
      return uri;
    }

    //get the parent of the parent
    parent = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *>(parent->getParentNode());
  }

  return textEmpty;
}

const XMLCh *Context::getInheritedNSPrefix(const XMLCh *uri) const
{
  //possible optimization - create a hash of prefix to URI once rather
  //then lookup every time

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *parent = v2NSUtils::returnOwnerElement(_triggerContext);

  while(parent!=NULL && parent->getNodeType() != XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE) {

    XERCES_CPP_NAMESPACE_QUALIFIER DOMNamedNodeMap *nodeList = parent->getAttributes();

    for(unsigned int i = 0;i < nodeList->getLength();i++) {
      XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *attr = nodeList->item(i);

      if(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(attr->getNamespaceURI(),textHttpXmlns)==0) {

	if(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(attr->getNodeValue(),uri)==0) {
	  if(attr->getPrefix() != 0) {
	    return attr->getLocalName();
	  }
	}

      }
    }

    //get the parent of the parent
    parent = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *>(parent->getParentNode());
  }

  return textEmpty;
}

const XMLCh *Context::getInheritedDefaultNS() const
{
  //possible optimization - create a hash of prefix to URI once rather
  //then lookup every time

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *parent = v2NSUtils::returnOwnerElement(_triggerContext);

  while(parent!=NULL && parent->getNodeType() != XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE) {
    const XMLCh *uri = parent->getAttributeNS(textHttpXmlns, textXmlns);

    if (uri != 0 && *uri != 0) {
      return uri;
    }

    //get the parent of the parent
    parent = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *>(parent->getParentNode());
  }

  return textEmpty;
}

bool Context::getCreativeMode() const
{
  return _creativeMode;
}

void Context::setCreativeMode(bool creative)
{
  _creativeMode = creative;
}

bool Context::isOrderedResultType() const
{
  return 
	_resultType==XPathResult::ORDERED_NODE_ITERATOR_TYPE ||
    _resultType==XPathResult::ORDERED_NODE_SNAPSHOT_TYPE ||
    _resultType==XPathResult::FIRST_ORDERED_NODE_TYPE;
}
