/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "DataItemBool.hpp"
#include "DataItem.hpp"
#include "DeadNodeList.hpp"
#include "../DOMutils/v2Utils.hpp"
#include "../defs/NumUtils.hpp"
#include <assert.h>
#include "../exceptions/DataItemException.hpp"
#include <xercesc/util/XMLUniDefs.hpp>
#include <xercesc/util/Janitor.hpp>
#include <memory>

#ifdef TEST
#include "DataItemNodeList.hpp"
#include "DataItemNumeric.hpp"
#include "DataItemString.hpp"
#endif

// "true"
static const XMLCh textTrue[] =
{
   XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_u, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

// "false"
static const XMLCh textFalse[] =
{
   XERCES_CPP_NAMESPACE_QUALIFIER chLatin_f, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_l, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

DataItemBool::DataItemBool(bool b)
   : _value(b)
{
  setType(DataItem::DATABOOLEAN);
}

DataItemBool::DataItemBool(const DataItemBool& rhs)
   : DataItem(rhs), _value(rhs._value)
{
}

DataItemBool::~DataItemBool()
{
  //no-op
}

bool DataItemBool::asBool(const Context *context) const
{
  return _value;
}

DataItem *DataItemBool::asSimpleDataItem(const Context *context) const
{
  return new DataItemBool(*this);
}

const XMLCh *DataItemBool::asString(const Context *context) const
{
  if(_value) {
    return textTrue;
  }
  else {
    return textFalse;
  }
}

XSFloat DataItemBool::asNum(const Context *context) const
{


  if(_value) {

    return 1;
  }
  else {

    return 0;
  }
}

DeadNodeList *DataItemBool::asNodeList(const Context *context, const bool singleNode) const
{
  throw DataItemException("DataItemBool::asNodeList", 
			  "Tried to convert from a boolean to a nodelist");

  return new DeadNodeList();
}

bool DataItemBool::isEqual(const DataItem *other,
			   const Context *context) const
{
  DataItem::whichType otherType = other->getType();

  bool equal = false;

  switch(otherType) {
  case  DataItem::STRINGLITERAL:
  case  DataItem::NUMERICLITERAL:
  case  DataItem::DATABOOLEAN:
    equal = (asBool(context) == other->asBool(context));
    break;
  case  DataItem::NODELIST:
  case  DataItem::NAVIGATION: {
    auto_ptr<DeadNodeList> list(other->asNodeList(context, false));

    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){

      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      bool empty = otherString.get()==NULL || otherString[0]==XERCES_CPP_NAMESPACE_QUALIFIER chNull;
      if(asBool(context) == !empty){
	equal = true;
	break;
      }
    }
    break;
  }
  case  DataItem::FUNCTION:
  case  DataItem::VARIABLE: {
     auto_ptr<DataItem> simpleData
        = static_cast< auto_ptr<DataItem> >(other->asSimpleDataItem(context));
     equal = isEqual(simpleData.get(), context);
     break;
  }
  default:
    assert(false);
    throw DataItemException("DataItemBool::isEqual", "enum case not dealt with");
  }


  return equal;
}

bool DataItemBool::isLessThan(const DataItem *other,
			      const Context *context) const
{

  DataItem::whichType otherType = other->getType();

  bool lessthan = false;

  switch(otherType) {
  case  DataItem::STRINGLITERAL:
  case  DataItem::NUMERICLITERAL:
  case  DataItem::DATABOOLEAN:
    lessthan = NumUtils::isLessThan(asNum(context), other->asNum(context));
    break;
  case  DataItem::NODELIST:
  case  DataItem::NAVIGATION: {
    auto_ptr<DeadNodeList> list(other->asNodeList(context, false));
    XSFloat myValue = asNum(context);

    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){

      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      XSFloat answer = NumUtils::stringAsDouble(otherString.get());
      if(NumUtils::isLessThan(myValue, answer)) {
	lessthan = true;
	break;
      }

    }
    break;
  }
  case  DataItem::VARIABLE:
  case  DataItem::FUNCTION: {
    auto_ptr<DataItem> simpleData
       = static_cast< auto_ptr<DataItem> >(other->asSimpleDataItem(context));
    lessthan = isLessThan(simpleData.get(), context);
    break;
  }
  default:
    assert(false);
    throw DataItemException("DataItemBool::isLessThan", "enum case not dealt with");
  }

  return lessthan;
}

bool DataItemBool::isGreaterThan(const DataItem *other,
				 const Context *context) const
{

  DataItem::whichType otherType = other->getType();

  bool greaterthan = false;

  switch(otherType) {
  case  DataItem::STRINGLITERAL:
  case  DataItem::NUMERICLITERAL:
  case  DataItem::DATABOOLEAN:
    greaterthan = NumUtils::isGreaterThan(asNum(context), other->asNum(context));
    break;
  case  DataItem::NODELIST:
  case  DataItem::NAVIGATION: {
    auto_ptr<DeadNodeList> list(other->asNodeList(context, false));
    XSFloat myValue = asNum(context);

    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){

      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      XSFloat answer = NumUtils::stringAsDouble(otherString.get());
      if(NumUtils::isGreaterThan(myValue, answer)) {
	greaterthan = true;
	break;
      }

    }
    break;
  }
  case  DataItem::VARIABLE:
  case  DataItem::FUNCTION: {
    auto_ptr<DataItem> simpleData
       = static_cast< auto_ptr<DataItem> >(other->asSimpleDataItem(context));
    greaterthan = isGreaterThan(simpleData.get(), context);
    break;
  }
  default:
    assert(false);
    throw DataItemException("DataItemBool::isGreaterThan", "enum case not dealt with");
  }

  return greaterthan;
}

#ifdef TEST
// Test functions

#include "../testharness/TestFunction.hpp"
#include "../DOMutils/XStr.hpp"

void DataItemBool::setUp()
{

  // Nothing to do

}

DataItemBool::DataItemBool(std::string name)
{

  setType(DataItem::DATABOOLEAN);

}

void DataItemBool::testAsBool()
{

  Context c;
  Context *context = &c;

  _value = true;
  assert(asBool(context) == true);

  _value = false;
  assert(asBool(context) == false);


}

void DataItemBool::testAsString()
{

  Context c;
  Context *context = &c;
 
  _value = false;
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("false"))==0);

  _value = true;
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("true"))==0);


}

void DataItemBool::testAsInt() {

  Context c;
  Context *context = &c;

  _value = true;
  assert(asInt(context) == 1);

  _value = false;
  assert(asInt(context) == 0);


}


void DataItemBool::testAsNum()
{

  Context c;
  Context *context = &c;

  _value = false;
  assert(asNum(context) == 0);

  _value = true;
  assert(asNum(context) == 1);


}

void DataItemBool::testAsNodeList()
{

  Context c;
  Context *context = &c;

  _value = false;
  bool caught = false;
  try {
    asNodeList(context, false);
  }
  catch(const DataItemException&) {
    caught = true;
  }
  assert(caught);

  _value = true;
  caught = false;
  try {
    asNodeList(context, false);
  }
  catch(const DataItemException&) {
    caught = true;
  }
  assert(caught);


}

void DataItemBool::testEquals()
{

  Context c;
  Context *context = &c;

  _value = true;
  auto_ptr<DataItem> booltrue(new DataItemBool(true));
  assert(isEqual(booltrue.get(), context));

  auto_ptr<DataItem> boolfalse(new DataItemBool(false));
  assert(!isEqual(boolfalse.get(), context));

  _value = false;
  assert(isEqual(boolfalse.get(), context));

  auto_ptr<DataItem> numzero(new DataItemNumeric(0));
  assert(isEqual(numzero.get(), context));

  auto_ptr<DataItem> numone(new DataItemNumeric(1));
  assert(!isEqual(numone.get(), context));

  _value = true;
  assert(isEqual(numone.get(), context));
  assert(!isEqual(numzero.get(), context));

  auto_ptr<DataItem> numrandom(new DataItemNumeric(487.83));
  assert(isEqual(numrandom.get(), context));

  auto_ptr<DataItem> stringfrank(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frank")));
  assert(isEqual(stringfrank.get(), context));

  auto_ptr<DataItem> stringblank(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("")));
  assert(!isEqual(stringblank.get(), context));

  _value = false;
  assert(isEqual(stringblank.get(), context));

  auto_ptr<DataItem> stringbob(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("bob bob bob")));
  assert(!isEqual(stringbob.get(), context));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr1 = data->createAttribute( X("one") );
  attr1->setNodeValue(X("jiminy"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr2 = data->createAttribute( X("two") );
  attr2->setNodeValue(X("cricket"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attrblank = data->createAttribute( X("three") );
  attrblank->setNodeValue(X(""));

  DeadNodeList *vec1= new DeadNodeList();
  DeadNodeList *vec2= new DeadNodeList();
  vec1->push_back(attr1);
  vec1->push_back(attrblank);
  vec2->push_back(attr2);
  vec2->push_back(attrblank);
  auto_ptr<DataItem> nodejiminy(new DataItemNodeList(vec1));
  auto_ptr<DataItem> nodecricket(new DataItemNodeList(vec2));

  assert(isEqual(nodecricket.get(), context));

  _value = true;
  assert(isEqual(nodejiminy.get(), context));

  /*
    true==true
    false==false
    true!=false

    true==1
    false==0
    true!=0
    false!=1
    true==487.83

    true=="frank"
    false==""
    true!=""
    false!="bob bob bob"
    
    true==("jiminy","")
    false==("cricket","")
   */

}

void DataItemBool::testLessThan()
{

  Context c;
  Context *context = &c;

  _value = true;
  auto_ptr<DataItem> booltrue(new DataItemBool(true));
  assert(!isLessThan(booltrue.get(), context));

  auto_ptr<DataItem> boolfalse(new DataItemBool(false));
  assert(!isLessThan(boolfalse.get(), context));

  auto_ptr<DataItem> num(new DataItemNumeric(54.3));
  assert(isLessThan(num.get(), context));

  auto_ptr<DataItem> numone(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("-32")));
  assert(!isLessThan(numone.get(), context));

  /*
    true !< true
    true !< false
    true < 54.3
    true !< "-32"
  */

}

void DataItemBool::testGreaterThan()
{

  Context c;
  Context *context = &c;

  _value = true;
  auto_ptr<DataItem> booltrue(new DataItemBool(true));
  assert(!isGreaterThan(booltrue.get(), context));

  auto_ptr<DataItem> boolfalse(new DataItemBool(false));
  assert(isGreaterThan(boolfalse.get(), context));

  auto_ptr<DataItem> num(new DataItemNumeric(54.3));
  assert(!isGreaterThan(num.get(), context));

  auto_ptr<DataItem> numone(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("-32")));
  assert(isGreaterThan(numone.get(), context));

  /*
    true !> true
    true > false
    true !> 54.3
    true > "-32"
  */
}

#endif
