/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include <algorithm>
#include "DataItemNav.hpp"
#include "../dataItem/DeadNodeList.hpp"
#include "DataItemString.hpp"
#include "DataItemNodeList.hpp"
#include "../navigation/SelfAxis.hpp"
#include "../context/Context.hpp"
#include <memory>

DataItemNav::DataItemNav()
  : _initialNavigation(NULL)
{
  setType(DataItem::NAVIGATION);
  _gotoRoot = false;
  _initialContext = CURRENT;
  _lastString = NULL;
}

DataItemNav::DataItemNav(DataItem *initialNavigation)
  : _initialNavigation(initialNavigation)
{
  setType(DataItem::NAVIGATION);
  _gotoRoot = false;
  _initialContext = CURRENT;
  _lastString = NULL;
}

DataItemNav::~DataItemNav()
{
  delete[] (XMLCh *)_lastString;
  _lastString=NULL;
  delete _initialNavigation, _initialNavigation=NULL;

  for(list<NavStep*>::iterator stepI = _steps.begin();
      stepI != _steps.end();
      ++stepI)
  {
    delete *stepI;
  }
}

bool DataItemNav::asBool(const Context *context) const
{
  auto_ptr<DataItem> simpleData = static_cast< auto_ptr<DataItem> >(asSimpleDataItem(context));
  return simpleData->asBool(context);
}

const XMLCh *DataItemNav::asString(const Context *context) const
{
  auto_ptr<DataItem> simpleData = static_cast< auto_ptr<DataItem> >(asSimpleDataItem(context));

  // Careful for exceptions
  delete[] (XMLCh *)_lastString;
  _lastString = NULL;
  _lastString = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(simpleData->asString(context));
  return _lastString;
}

XSFloat DataItemNav::asNum(const Context *context) const
{
  auto_ptr<DataItem> simpleData = static_cast< auto_ptr<DataItem> >(asSimpleDataItem(context));
  return simpleData->asNum(context);
}

DeadNodeList *DataItemNav::asNodeList(const Context *context, const bool singleNode) const
{
  return evalTree(context, singleNode);
}

bool DataItemNav::isEqual(const DataItem *other,
			  const Context *context) const
{
  auto_ptr<DataItem> simpleData = static_cast< auto_ptr<DataItem> >(asSimpleDataItem(context));
  return simpleData->isEqual(other, context);
}

bool DataItemNav::isLessThan(const DataItem *other,
			     const Context *context) const
{
  auto_ptr<DataItem> simpleData = static_cast< auto_ptr<DataItem> >(asSimpleDataItem(context));
  return simpleData->isLessThan(other, context);
}

bool DataItemNav::isGreaterThan(const DataItem *other,
				const Context *context) const
{
  auto_ptr<DataItem> simpleData = static_cast< auto_ptr<DataItem> >(asSimpleDataItem(context));
  return simpleData->isGreaterThan(other, context);
}

DataItem *DataItemNav::asSimpleDataItem(const Context *context) const
{
  return new DataItemNodeList(evalTree(context, false));
}

void DataItemNav::addPredicate(PredicateStep *step)
{
  if(_steps.size() == 0) {
    NavStep *newStep = new SelfAxis();
    newStep->setTypeWildcard();
    newStep->setNameWildcard();
    newStep->setNamespaceWildcard();
    _steps.push_back(newStep);
  }
  _steps.back()->addPredicate(step);
}

void DataItemNav::addPredicates(const vector<PredicateStep*> &predSteps)
{
  if(_steps.size() == 0) {
    NavStep *newStep = new SelfAxis();
    newStep->setTypeWildcard();
    newStep->setNameWildcard();
    newStep->setNamespaceWildcard();
    _steps.push_back(newStep);
  }

  vector<PredicateStep*>::const_iterator i;

  for(i=predSteps.begin();i!=predSteps.end();++i) {
    _steps.back()->addPredicate(*i);
  }
}

void DataItemNav::addStep(NavStep *step)
{
  _steps.push_back(step);
}

void DataItemNav::addStepFront(NavStep *step)
{
  _steps.push_front(step);
}

DeadNodeList *DataItemNav::evalTree(const Context *context, const bool singleNode) const
{
  if(context->getCreativeMode()) {
    return doCreativeNavigation(context, singleNode);
  }
  else {
    return doNavigation(context, singleNode);
  }
}

void DataItemNav::addNavSteps(DataItemNav *nav)
{
  for(list<NavStep*>::const_iterator i = nav->_steps.begin();
      i != nav->_steps.end(); ++i) {
    _steps.push_back(*i);
  }

  // Ensure steps are removed from source nav, so they don't get deleted twice
  nav->_steps.clear();
}

void DataItemNav::setGotoRootFirst(bool gotoRoot)
{
  _gotoRoot = gotoRoot;
}


void DataItemNav::setInitialContext(contextType cont)
{
  _initialContext = cont;
}

contextType DataItemNav::getInitialContext(void) const
{
  return _initialContext;
}

DeadNodeList *DataItemNav::setUpDoNavigation(const Context *context) const
{
  DeadNodeList *candidateList = new DeadNodeList();

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *initialContext = NULL;

  switch(_initialContext) {
  case INSTRUCTION:{
    return candidateList;
  }
  case DATA:{
    return candidateList;
  }
  case OLD:{
    return candidateList;
  }
  case CURRENT:{
    initialContext = context->getCurrentContext();
    break;
  }
  }

  if(_gotoRoot && initialContext!=NULL) {
    XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *temp = initialContext->getOwnerDocument();
    if(temp!=NULL) {
      initialContext = temp;
    }
  }

  candidateList->push_back(initialContext);

  return candidateList;
}

DeadNodeList *DataItemNav::doNavigation(const Context *context, const bool singleNode) const
{
  DeadNodeList *candidateList = NULL;

  if(_initialNavigation) {
    candidateList = _initialNavigation->asNodeList(context, false)->clone(false);
  }
  else {
    candidateList = setUpDoNavigation(context);
  }

  if(!candidateList->empty()) {
    Context newcontext(*context);
    newcontext.prepareForLoop();

	DeadNodeList tmpList;
	DeadNodeList *l1= candidateList;
	DeadNodeList *l2= &tmpList;
    list<NavStep*>::const_iterator s = _steps.begin();
    while (s != _steps.end()) {
      list<NavStep*>::const_iterator t = s;
      bool firstMatch = (++t == _steps.end()) && singleNode;
      (*(s++))->doStep(l1, &newcontext, false, firstMatch, l2);
	  std::swap(l1, l2);
	  l2->clear();
    }
	if(l1!=candidateList)
	{
		candidateList->addList(l1);
	}
    candidateList->removeDuplicates(context->isOrderedResultType());
  }
  return candidateList;
}

DeadNodeList *DataItemNav::doCreativeNavigation(const Context *context, const bool singleNode) const
{
  //Get the initial nodelist, if any from the stored dataitem
  DeadNodeList *candidateList = NULL;

  if(_initialNavigation) {
    candidateList = _initialNavigation->asNodeList(context, false)->clone(false);
  }
  else {
    candidateList = setUpDoNavigation(context);
  }

  if(!candidateList->empty()) {

    Context newContext(*context);
    newContext.prepareForLoop();

    if( _steps.size() != 0 ) { // You can have a navigation with no steps (like "\")
      //find the last element
      list<NavStep*>::const_iterator rlast = _steps.end();
      list<NavStep*>::const_iterator last = --rlast;
      //do the normal steps
	  DeadNodeList tmpList;
	  DeadNodeList *l1= candidateList;
	  DeadNodeList *l2= &tmpList;
      for(list<NavStep*>::const_iterator s = _steps.begin(); s != last; ++s) {
        (*s)->doStep(l1, &newContext, false, false, l2); //pass false for singleNode since creative mode ignores it
	    std::swap(l1, l2);
	    l2->clear();
      }

      //ensure there are items in the candidate list
      if(l1->getLength()) {
        (*last)->doStep(l1, &newContext, true, false, l2); //pass false for singleNode since creative mode ignores it
	    std::swap(l1, l2);
      }
	  if(l1!=candidateList)
	  {
		candidateList->addList(l1);
	  }
    }
    candidateList->removeDuplicates(context->isOrderedResultType());
  }

  return candidateList;
}


#ifdef TEST
// (Fake) Test functions

#include "../testharness/TestFunction.hpp"

void DataItemNav::setUp()
{

  // Nothing to do

}

DataItemNav::DataItemNav(std::string name) : _initialNavigation(new DataItemNodeList(new DeadNodeList()))
{


}

void DataItemNav::testAsBool()
{


}

void DataItemNav::testAsString()
{


}

void DataItemNav::testAsNum()
{


}

void DataItemNav::testAsNodeList()
{


}

void DataItemNav::testEquals()
{


}

void DataItemNav::testLessThan()
{


}

void DataItemNav::testGreaterThan()
{


}

#endif
