/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _DataItemNav_HPP
#define _DataItemNav_HPP

#include <config.h>

#include "DataItem.hpp"
#include "DataItemNodeList.hpp"
#include "../navigation/PredicateStep.hpp"
#include "../navigation/NavStep.hpp"
#include "../functionAPI/FunctionLookup.hpp"

#include <list>

typedef enum {
  INSTRUCTION,
  DATA,
  OLD,
  CURRENT
} contextType;

///this class calculates a node list based on a context
class PATHAN_EXPORT DataItemNav : public DataItem
{
public:
  DataItemNav();
  DataItemNav(DataItem *initialNavigation);
  ~DataItemNav();

  ///execute the tree stucture and call this method in the base class
  bool asBool(const Context *context) const;
  ///execute the tree stucture and call this method in the base class
  const XMLCh *asString(const Context *context) const;
  ///execute the tree stucture and call this method in the base class
  XSFloat asNum(const Context *context) const;
  ///execute the tree stucture and call this method in the base class
  DeadNodeList *asNodeList(const Context *context, const bool singleNode) const;

  DataItem *asSimpleDataItem(const Context *context) const;

  ///execute the tree stucture and call this method in the base class
  bool isEqual(const DataItem *other, const Context *context) const;
  ///execute the tree stucture and call this method in the base class
  bool isLessThan(const DataItem *other, const Context *context) const;
  ///execute the tree stucture and call this method in the base class
  bool isGreaterThan(const DataItem *other, const Context *context) const;

  /// Execute the navigation, returning the nodes in _nodeList.
  DeadNodeList *doNavigation(const Context *context, const bool singleNode) const;

  /** Execute the navigation in creative mode (nodes may be created if
      they do not exist), returning the nodes in _nodeList. **/
  DeadNodeList *doCreativeNavigation(const Context *context, const bool singleNode) const;

  /** Add a new NavigationStep to the sequence of navigation steps to
      be performed. This operation is used by parse tree classes to
      turn themselves into an optimised, reusable navigation. **/
  void addStep(NavStep *step);

  /** Add a new NavigationStep to the start of the sequence of
      navigation steps to be performed.**/
  void addStepFront(NavStep *step);

  ///adds a predicate to the current step of the navigation
  void addPredicate(PredicateStep *step);

  ///adds several predicates to the current step of the navigation
  void addPredicates(const vector<PredicateStep*> &predSteps);

  ///this takes the navsteps out of item and inserts them into this dataitem
  void addNavSteps(DataItemNav *nav);

  ///returns the nav steps assciated with this item
  list<const NavStep*> getNavSteps() const;

  ///set the "go to root of tree first" flag.
  void setGotoRootFirst(bool gotoRoot);

  ///sets the node that will be used for the starting point of the navigation
  void setInitialContext(contextType cont);
  ///sets the node that will be used for the starting point of the navigation
  contextType getInitialContext(void) const;

private:

#ifdef TEST
  ///Dummy constructor for test suite
  DataItemNav(std::string name);
  ///Test setup
  void setUp();
  ///Test function
  void testAsBool();
  ///Test function
  void testAsString();
  ///Test function
  void testAsNum();
  ///Test function
  void testAsNodeList();
  ///Test function
  void testAsInt();
  ///Test equals
  void testEquals();
  ///Test less than
  void testLessThan();
  ///Test greater than
  void testGreaterThan();
#endif

private:
  ///set up for navigation
  DeadNodeList *setUpDoNavigation(const Context *context) const;
  ///evaluate tree
  DeadNodeList *evalTree(const Context *context, const bool singleNode) const;

  bool _gotoRoot;
  contextType _initialContext;

  DataItem *_initialNavigation;

  mutable const XMLCh *_lastString;
  mutable DeadNodeList *_lastNodeList;

  //list of steps to be performed.
  list<NavStep*> _steps;
};

#endif


