/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "DataItemNodeList.hpp"

#include <assert.h>
#include <math.h>
#include "DataItem.hpp"
#include "../DOMutils/v2Utils.hpp"
#include "../defs/NumUtils.hpp"
#include <xercesc/util/XMLUniDefs.hpp>
#include <xercesc/util/Janitor.hpp>
#include <memory>

#include "DataItemString.hpp"
#include "DataItemBool.hpp"
#include "DataItemNumeric.hpp"

// ""
static const XMLCh textEmpty[] =
{
   XERCES_CPP_NAMESPACE_QUALIFIER chNull
};

DataItemNodeList::DataItemNodeList(DeadNodeList *v)
{
  _nodeList = v;
  _cachedString = NULL;
  setType(DataItem::NODELIST);
}

DataItemNodeList::DataItemNodeList(const DataItemNodeList& rhs)
   : DataItem(rhs), _nodeList(rhs._nodeList->clone(false)), _cachedString(NULL)
{
}

DataItemNodeList::~DataItemNodeList()
{
   delete _nodeList, _nodeList=NULL;
   delete[] (XMLCh *)_cachedString;
   _cachedString=NULL;
}

bool DataItemNodeList::asBool(const Context *context) const
{
  if(_nodeList->getLength()){
    return true;
  }
  else{
    return false;
  }
}

const XMLCh *DataItemNodeList::asString(const Context *context) const
{
  if(_nodeList->getLength()) {
    if(!_cachedString)
    {
      _cachedString = v2Utils::getStringFromNode(_nodeList->item(0));
    }
    return _cachedString;
  }
  else {
    return textEmpty;
  }
}

DeadNodeList *DataItemNodeList::asNodeList(const Context *context, const bool singleNode) const
{
	return _nodeList->clone(false);
}

XSFloat DataItemNodeList::asNum(const Context *context) const
{
  return NumUtils::stringAsDouble(asString(context));
}

DataItem *DataItemNodeList::asSimpleDataItem(const Context *context) const
{
  return new DataItemNodeList(*this);
}

bool DataItemNodeList::isEqual(const DataItem *other,
			       const Context *context) const
{
  DataItem::whichType otherType = other->getType();

  bool equal = false;

  switch(otherType) {

    // search the node list for a string that is the same as the other
  case  DataItem::STRINGLITERAL: {
    auto_ptr<DeadNodeList> list(asNodeList(context, false));
    const XMLCh *myValue = other->asString(context);

    if(list->getLength()==0 && (myValue==NULL || myValue[0]==0)) {

      return true;
    }

    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){
      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      equal = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(otherString.get(),myValue)==0;
      if(equal) {
	break;
      }
    }
    break;
  }

  // search the node list for a number thats the same
  case  DataItem::NUMERICLITERAL:{

    auto_ptr<DeadNodeList> list (asNodeList(context, false));
    XSFloat myValue = other->asNum(context);

    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){

      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      XSFloat answer = NumUtils::stringAsDouble(otherString.get());
      if(NumUtils::isEqual(myValue, answer)) {
	equal = true;
	break;
      }

    }

    break;
  }

  // if the string contents of any matches the string content of any
  // node in the other nodelist then true is returned
  case  DataItem::NODELIST:
  case  DataItem::NAVIGATION: {
    auto_ptr<DeadNodeList> list1(asNodeList(context, false));
    auto_ptr<DeadNodeList> list2(other->asNodeList(context, false));
    equal = v2Utils::compareNodeLists(list1.get(), list2.get());
    break;
  }
  case  DataItem::DATABOOLEAN: {
    auto_ptr<DeadNodeList> list(asNodeList(context, false));
    bool myValue = other->asBool(context);

    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){

      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      bool empty = otherString.get()==NULL || otherString[0]==XERCES_CPP_NAMESPACE_QUALIFIER chNull;

      //if the nodeString length is 0 then it is false
      if(myValue == !empty) {
	equal = true;
	break;
      }
    }
    break;
  }
  case DataItem::FUNCTION:
  case DataItem::VARIABLE: {
    auto_ptr<DataItem> simpleData
       = static_cast< auto_ptr<DataItem> >(other->asSimpleDataItem(context));
    equal = isEqual(simpleData.get(), context);
    break;
  }
  default:
    assert(false);
    throw DataItemException("DataItemNodeList::isEqual", "enum case not dealt with");
  }


  return equal;
}

bool DataItemNodeList::isLessThan(const DataItem *other,
				  const Context *context) const
{

  DataItem::whichType otherType = other->getType();

  bool lessthan = false;

  switch(otherType) {

    // search the node list for a value that is less than other->asNum(context)
  case  DataItem::NUMERICLITERAL:
  case  DataItem::DATABOOLEAN:
  case  DataItem::STRINGLITERAL: {
    auto_ptr<DeadNodeList> list(asNodeList(context, false));
    XSFloat myValue = other->asNum(context);


    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){

      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      XSFloat answer = NumUtils::stringAsDouble(otherString.get());
      if(NumUtils::isLessThan(answer, myValue)) {
	lessthan = true;
	break;
      }
    }
    break;
  }
  // compare all values in both lists... tested in v2Utils, not here.
  case  DataItem::NODELIST:
  case  DataItem::NAVIGATION: {
    auto_ptr<DeadNodeList> list1(asNodeList(context, false));
    auto_ptr<DeadNodeList> list2(other->asNodeList(context, false));
    lessthan = v2Utils::compareNodeListsLessThan(list1.get(), list2.get());
    break;
  }
  case  DataItem::VARIABLE:
  case  DataItem::FUNCTION: {
    auto_ptr<DataItem> simpleData
       = static_cast< auto_ptr<DataItem> >(other->asSimpleDataItem(context));
    lessthan = isLessThan(simpleData.get(), context);
    break;
  }
  default:
    assert(false);
    throw DataItemException("DataItemNodeList::isLessThan", "enum case not dealt with");
  }

  return lessthan;
}

bool DataItemNodeList::isGreaterThan(const DataItem *other,
				     const Context *context) const
{

  DataItem::whichType otherType = other->getType();

  bool greaterthan = false;

  switch(otherType) {

    // search the node list for a value that is less than other.asNum(context)
  case  DataItem::NUMERICLITERAL:
  case  DataItem::DATABOOLEAN:
  case  DataItem::STRINGLITERAL: {
    auto_ptr<DeadNodeList> list(asNodeList(context, false));
    XSFloat myValue = other->asNum(context);

    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){

      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      XSFloat answer = NumUtils::stringAsDouble(otherString.get());
      if(NumUtils::isGreaterThan(answer, myValue)) {
	greaterthan = true;
	break;
      }
    }
    break;
  }
  // compare all values in both lists... tested in v2Utils, not here.
  case  DataItem::NODELIST:
  case  DataItem::NAVIGATION: {
    auto_ptr<DeadNodeList> list1(asNodeList(context, false));
    auto_ptr<DeadNodeList> list2(other->asNodeList(context, false));
    greaterthan = v2Utils::compareNodeListsGreaterThan(list1.get(), list2.get());
    break;
  }
  case  DataItem::VARIABLE:
  case  DataItem::FUNCTION: {
    auto_ptr<DataItem> simpleData
       = static_cast< auto_ptr<DataItem> >(other->asSimpleDataItem(context));
    greaterthan = isGreaterThan(simpleData.get(), context);
    break;
  }
  default:
    assert(false);
    throw DataItemException("DataItemNodeList::isGreaterThan", "enum case not dealt with");
  }

  return greaterthan;
}

#ifdef TEST
// Test functions

#include "../testharness/TestFunction.hpp"
#include "../DOMutils/XStr.hpp"

void DataItemNodeList::setUp()
{

  // Nothing to do

}

DataItemNodeList::DataItemNodeList(std::string name)
   : DataItem( name ), _nodeList(new DeadNodeList), _cachedString(NULL)
{
  setType(DataItem::NUMERICLITERAL);
}

void DataItemNodeList::testAsBool()
{

  Context c;
  Context *context = &c;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  assert(asBool(context) == false);

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createElement( X("node_test") ));
  _nodeList->push_back(node);
  assert(asBool(context) == true);

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node2 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createElement( X("node_test2") ));
  _nodeList->push_back(node2);

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node3 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createElement( X("node_test3") ));
  _nodeList->push_back(node3);

  assert(asBool(context) == true);

}

void DataItemNodeList::testAsString()
{
  Context c;
  Context *context = &c;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),(X("")))==0);
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("node test") ));

  _nodeList->push_back(node);
  delete _cachedString;
  _cachedString = NULL;
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),(X("node test")))==0);
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node2 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("toshmonster") ));
  _nodeList->push_back(node2);

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node3 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("noggin") ));
  _nodeList->push_back(node3);
  delete _cachedString;
  _cachedString = NULL;
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),(X("node test")))==0);
  _nodeList->clear();
  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *node4 = data->createElement(X("foo"));
  node4->appendChild(node3);
  node4->appendChild(node2);
  _nodeList->push_back(node4);
  delete _cachedString;
  _cachedString = NULL;
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),(X("noggintoshmonster")))==0);
  _nodeList->clear();
  node4->setAttribute(X("jim"),X("frank"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr = node4->getAttributeNode(X("jim"));
  _nodeList->push_back(attr);
  delete _cachedString;
  _cachedString = NULL;
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),(X("frank")))==0);
}

void DataItemNodeList::testAsNum()
{
  Context c;
  Context *context = &c;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  delete _cachedString;
  _cachedString=NULL;
  assert(NumUtils::isNaN(asNum(context))); //NaN

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("56.45") ));
  _nodeList->push_back(node);

  delete _cachedString;
  _cachedString=NULL;
  assert(NumUtils::isEqual(asNum(context), 56.45L));

  _nodeList->item(0)->setNodeValue(X("ssd20"));
  delete _cachedString;
  _cachedString=NULL;
  assert(NumUtils::isNaN(asNum(context)));//NaN

  _nodeList->item(0)->setNodeValue(X("sd20.2df"));
  delete _cachedString;
  _cachedString=NULL;
  assert(NumUtils::isNaN(asNum(context)));//NaN

  _nodeList->item(0)->setNodeValue(X("34.2sd20.2df"));
  delete _cachedString;
  _cachedString=NULL;
  assert(NumUtils::isNaN(asNum(context)));//NaN

  _nodeList->item(0)->setNodeValue(X("-89.6 453sd20.2df"));
  delete _cachedString;
  _cachedString=NULL;
  assert(NumUtils::isNaN(asNum(context)));//NaN

}

void DataItemNodeList::testAsInt()
{
  Context c;
  Context *context = &c;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  delete _cachedString;
  _cachedString=NULL;
  assert(asInt(context) == 0);

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("56.45") ));
  _nodeList->push_back(node);
  delete _cachedString;
  _cachedString=NULL;
  assert(asInt(context) == 56);

  _nodeList->item(0)->setNodeValue(X("56.55"));
  delete _cachedString;
  _cachedString=NULL;
  assert(asInt(context) == 57);

  _nodeList->item(0)->setNodeValue(X("3"));
  delete _cachedString;
  _cachedString=NULL;
  assert(asInt(context) == 3);

  _nodeList->item(0)->setNodeValue(X("ssd20"));
  delete _cachedString;
  _cachedString=NULL;
  assert(asInt(context) == 0);

  _nodeList->item(0)->setNodeValue(X("sd20.2df"));
  delete _cachedString;
  _cachedString=NULL;
  assert(asInt(context) == 0);

  _nodeList->item(0)->setNodeValue(X("34.2sd20.2df"));
  delete _cachedString;
  _cachedString=NULL;
  assert(asInt(context) == 0);

  _nodeList->item(0)->setNodeValue(X("-89.6 453sd20.2df"));
  delete _cachedString;
  _cachedString=NULL;
  assert(asInt(context) == 0);

}


void DataItemNodeList::testAsNodeList()
{
  Context c;
  Context *context = &c;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  auto_ptr<DeadNodeList> nodelist1(asNodeList(context, false));
  assert(nodelist1->getLength() == 0);

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("56.45")));
  _nodeList->push_back(node);

  auto_ptr<DeadNodeList> nodelist2(asNodeList(context, false));
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(nodelist2->item(0)->getNodeValue(),X("56.45"))==0);
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node2 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("fred")));
  _nodeList->push_back(node2);

  auto_ptr<DeadNodeList> nodelist3(asNodeList(context, false));
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(nodelist3->item(1)->getNodeValue(),X("fred"))==0);
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node3 = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *> (data->createTextNode( X("bethusela")));
  _nodeList->push_back(node3);

  auto_ptr<DeadNodeList> nodelist4(asNodeList(context, false));
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(nodelist4->item(2)->getNodeValue(),X("bethusela"))==0);
}

// Nodelist==Nodelist is tested in v2Utils
void DataItemNodeList::testEquals()
{
  Context c;
  Context *context = &c;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr1 = data->createAttribute( X("one") );
  attr1->setNodeValue(X("jiminy"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr2 = data->createAttribute( X("two") );
  attr2->setNodeValue(X("cricket"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr3 = data->createAttribute( X("three") );
  attr3->setNodeValue(X("76.63"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr4 = data->createAttribute( X("four") );
  attr4->setNodeValue(X("54.4"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr5 = data->createAttribute( X("five") );
  attr5->setNodeValue(X("frank jim fred"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr6 = data->createAttribute( X("six") );
  attr6->setNodeValue(X(""));

  _nodeList->clear();
  _nodeList->push_back(attr1);
  _nodeList->push_back(attr6);
  auto_ptr<DataItem> booltrue(new DataItemBool(true));
  assert(isEqual(booltrue.get(), context));

  _nodeList->clear();
  _nodeList->push_back(attr2);
  _nodeList->push_back(attr6);
  auto_ptr<DataItem> boolfalse(new DataItemBool(false));
  assert(isEqual(boolfalse.get(), context));

  _nodeList->clear();
  _nodeList->push_back(attr1);
  _nodeList->push_back(attr3);
  auto_ptr<DataItem> numseventy(new DataItemNumeric(76.63));
  _cachedString=NULL;
  assert(isEqual(numseventy.get(), context));

  _nodeList->clear();
  _nodeList->push_back(attr4);
  _nodeList->push_back(attr2);
  auto_ptr<DataItem> numfifty(new DataItemNumeric(54.3));
  _cachedString=NULL;
  assert(!isEqual(numfifty.get(), context));

  _nodeList->clear();
  _nodeList->push_back(attr1);
  _nodeList->push_back(attr5);
  auto_ptr<DataItem> stringfrank(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frank jim fred")));
  _cachedString=NULL;
  assert(isEqual(stringfrank.get(), context));

  auto_ptr<DataItem> stringnull(new DataItemString(NULL));
  _cachedString=NULL;
  assert(!isEqual(stringnull.get(), context));

  _nodeList->clear();
  _nodeList->push_back(attr2);
  _nodeList->push_back(attr6);
  _cachedString=NULL;
  assert(isEqual(stringnull.get(), context));

  /*
    ("jiminy","")==true
    ("cricket","")==false

    ("jiminy", "76.63")==76.63
    ("54.4", "cricket")!=54.3

    ("jiminy", "frank jim fred")=="frank jim fred"
    ("jiminy", "frank jim fred")!=""
    ("cricket", "")==""

    Nodelist==Nodelist - tested in v2Utils
  */

}

void DataItemNodeList::testLessThan()
{
  Context c;
  Context *context = &c;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr1 = data->createAttribute( X("one") );
  attr1->setNodeValue(X("1"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr2 = data->createAttribute( X("two") );
  attr2->setNodeValue(X("2"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr3 = data->createAttribute( X("three") );
  attr3->setNodeValue(X("3"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr4 = data->createAttribute( X("four") );
  attr4->setNodeValue(X("4"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr5 = data->createAttribute( X("five") );
  attr5->setNodeValue(X("5"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr6 = data->createAttribute( X("six") );
  attr6->setNodeValue(X("6"));

  DeadNodeList *other1 = new DeadNodeList();
  _nodeList->clear();
  _nodeList->push_back(attr1);
  _nodeList->push_back(attr2);
  _nodeList->push_back(attr3);
  other1->push_back(attr2);
  auto_ptr<DataItem> otherlist(new DataItemNodeList(other1));
  assert(!isLessThan(otherlist.get(), context));

  _nodeList->clear();
  _nodeList->push_back(attr1);
  _nodeList->push_back(attr2);
  _nodeList->push_back(attr3);
  DeadNodeList *other2 = new DeadNodeList();
  other2->push_back(attr4);
  other2->push_back(attr5);
  other2->push_back(attr6);
  auto_ptr<DataItem> fourfivesix(new DataItemNodeList(other2));
  assert(isLessThan(fourfivesix.get(), context));

  _nodeList->clear();
  _nodeList->push_back(attr1);
  _nodeList->push_back(attr2);
  _nodeList->push_back(attr3);
  auto_ptr<DataItem> numfifty(new DataItemNumeric(54.3));
  _cachedString=NULL;
  assert(isLessThan(numfifty.get(), context));

  _nodeList->clear();
  _nodeList->push_back(attr6);
  auto_ptr<DataItem> numtwo(new DataItemNumeric(2));
  _cachedString=NULL;
  assert(!isLessThan(numtwo.get(), context));

  /*
    ("1", "2", "3") !< ("2")
    ("1", "2", "3") < ("4", "5", "6")
    ("1", "2", "3") < 54.3
    ("6") !< 2
  */

}

void DataItemNodeList::testGreaterThan()
{
  Context c;
  Context *context = &c;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr1 = data->createAttribute( X("one") );
  attr1->setNodeValue(X("1"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr2 = data->createAttribute( X("two") );
  attr2->setNodeValue(X("2"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr3 = data->createAttribute( X("three") );
  attr3->setNodeValue(X("3"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr4 = data->createAttribute( X("four") );
  attr4->setNodeValue(X("4"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr5 = data->createAttribute( X("five") );
  attr5->setNodeValue(X("5"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr6 = data->createAttribute( X("six") );
  attr6->setNodeValue(X("6"));

  DeadNodeList *other1 = new DeadNodeList;
  _nodeList->clear();
  _nodeList->push_back(attr1);
  _nodeList->push_back(attr2);
  _nodeList->push_back(attr3);
  other1->push_back(attr2);
  auto_ptr<DataItem> otherlist(new DataItemNodeList(other1));
  assert(!isGreaterThan(otherlist.get(), context));

  DeadNodeList *other2 = new DeadNodeList;
  _nodeList->push_back(attr1);
  _nodeList->push_back(attr2);
  _nodeList->push_back(attr3);
  other2->push_back(attr4);
  other2->push_back(attr5);
  other2->push_back(attr6);
  auto_ptr<DataItem> fourfivesix(new DataItemNodeList(other2));
  assert(!isGreaterThan(fourfivesix.get(), context));

  _nodeList->clear();
  _nodeList->push_back(attr1);
  _nodeList->push_back(attr2);
  _nodeList->push_back(attr3);
  auto_ptr<DataItem> numfifty(new DataItemNumeric(54.3));
  _cachedString=NULL;
  assert(!isGreaterThan(numfifty.get(), context));

  _nodeList->clear();
  _nodeList->push_back(attr6);
  auto_ptr<DataItem> numtwo(new DataItemNumeric(2));
  _cachedString=NULL;
  assert(isGreaterThan(numtwo.get(), context));

  /*
    ("1", "2", "3") !> ("2")
    ("1", "2", "3") !> ("4", "5", "6")
    ("1", "2", "3") !> 54.3
    ("6") > 2
  */

}


//*********************
#endif
