/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "DataItemNumeric.hpp"
#include "DataItem.hpp"
#include "DeadNodeList.hpp"
#include <assert.h>
#include "../exceptions/DataItemException.hpp"
#include "../defs/NumUtils.hpp"
#include <memory>
#include <xercesc/util/Janitor.hpp>

#ifdef WIN32
#define snprintf _snprintf
#endif

#ifdef TEST
#include "DataItemNodeList.hpp"
#include "DataItemBool.hpp"
#include "DataItemString.hpp"
#endif

DataItemNumeric::DataItemNumeric(XSFloat d)
{
  _value = d;
  _cachedString = NULL;
  setType(DataItem::NUMERICLITERAL);
}

DataItemNumeric::~DataItemNumeric()
{
   delete[] (XMLCh *)_cachedString;
   _cachedString=NULL;
}

DataItemNumeric::DataItemNumeric(const DataItemNumeric& rhs)
   : DataItem(rhs), _value(rhs._value), _cachedString(NULL)
{
}

DataItem *DataItemNumeric::asSimpleDataItem(const Context *context) const
{
  return new DataItemNumeric(*this);
}

bool DataItemNumeric::asBool(const Context *context) const
{
  if(_value == 0){

    return false;
  }
  else{

    return true;
  }
}

const XMLCh *DataItemNumeric::asString(const Context *context) const
{
  if(!_cachedString)
  {
     _cachedString = NumUtils::doubleAsString(_value);
  }
  return _cachedString;
}

XSFloat DataItemNumeric::asNum(const Context *context) const
{
  return _value;
}

DeadNodeList *DataItemNumeric::asNodeList(const Context *context, const bool singleNode) const
{
  throw DataItemException("DataItemNumeric::asNodeList", 
			  "Tried to convert from a number to a nodelist");

  return new DeadNodeList();
}

bool DataItemNumeric::isEqual(const DataItem *other,
			      const Context *context) const
{

  DataItem::whichType otherType = other->getType();

  bool equal = false;

  switch(otherType) {
  case  DataItem::STRINGLITERAL:
  case  DataItem::NUMERICLITERAL: {
    equal = NumUtils::isEqual(asNum(context), other->asNum(context));
    break;
  }
  case  DataItem::NODELIST:
  case  DataItem::NAVIGATION: {
    auto_ptr<DeadNodeList> list(other->asNodeList(context, false));
    XSFloat myValue = asNum(context);

    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){

      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      XSFloat answer = NumUtils::stringAsDouble(otherString.get());
      if(NumUtils::isEqual(myValue, answer)) {
	equal = true;
	break;
      }

    }
    break;
  }
  case  DataItem::DATABOOLEAN:
    equal = (asBool(context) == other->asBool(context));
    break;
  case  DataItem::FUNCTION:
  case  DataItem::VARIABLE: {
    auto_ptr<DataItem> simpleData
       = static_cast< auto_ptr<DataItem> >(other->asSimpleDataItem(context));
    equal = isEqual(simpleData.get(), context);
    break;
  }
  default:
    assert(false);
    throw DataItemException("DataItemNumeric::isEquals", "enum case not dealt with");
  }


  return equal;
}

bool DataItemNumeric::isLessThan(const DataItem *other,
				 const Context *context) const
{

  DataItem::whichType otherType = other->getType();

  bool lessthan = false;

  switch(otherType) {
  case  DataItem::STRINGLITERAL:
  case  DataItem::NUMERICLITERAL:
  case  DataItem::DATABOOLEAN:
    lessthan = NumUtils::isLessThan(asNum(context), other->asNum(context));
    break;
  case  DataItem::NODELIST:
  case  DataItem::NAVIGATION: {
    auto_ptr<DeadNodeList> list(other->asNodeList(context, false));
    XSFloat myValue = asNum(context);

    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){

      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      XSFloat answer = NumUtils::stringAsDouble(otherString.get());
      if(NumUtils::isLessThan(myValue, answer)) {
	lessthan = true;
	break;
      }

    }
    break;
  }
  case  DataItem::VARIABLE:
  case  DataItem::FUNCTION: {
    auto_ptr<DataItem> simpleData
       = static_cast< auto_ptr<DataItem> >(other->asSimpleDataItem(context));
    lessthan = isLessThan(simpleData.get(), context);
    break;
  }
  default:
    assert(false);
    throw DataItemException("DataItemNumeric::isLessThan", "enum case not dealt with");
  }


  return lessthan;
}

bool DataItemNumeric::isGreaterThan(const DataItem *other,
				    const Context *context) const
{

  DataItem::whichType otherType = other->getType();

  bool greaterthan = false;

  switch(otherType) {
  case  DataItem::STRINGLITERAL:
  case  DataItem::NUMERICLITERAL:
  case  DataItem::DATABOOLEAN:
    greaterthan = NumUtils::isGreaterThan(asNum(context), other->asNum(context));
    break;
  case  DataItem::NODELIST:
  case  DataItem::NAVIGATION: {
    auto_ptr<DeadNodeList> list(other->asNodeList(context, false));
    XSFloat myValue = asNum(context);

    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){

      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      XSFloat answer = NumUtils::stringAsDouble(otherString.get());
      if(NumUtils::isGreaterThan(myValue, answer)) {
	greaterthan = true;
	break;
      }

    }
    break;
  }
  case  DataItem::VARIABLE:
  case  DataItem::FUNCTION: {
    auto_ptr<DataItem> simpleData
       = static_cast< auto_ptr<DataItem> >(other->asSimpleDataItem(context));
    greaterthan = isGreaterThan(simpleData.get(), context);
    break;
  }
  default:
    assert(false);
    throw DataItemException("DataItemNumeric::isGreaterThan", "DataItemNumeric enum case not dealt with");
  }


  return greaterthan;
}

#ifdef TEST
// Test functions

#include "../testharness/TestFunction.hpp"
#include "../DOMutils/XStr.hpp"

void DataItemNumeric::setUp()
{

  // Nothing to do

}

DataItemNumeric::DataItemNumeric(std::string name)
{
  _value = 0.0;
  _cachedString = NULL;

  setType(DataItem::NUMERICLITERAL);

}

void DataItemNumeric::testAsBool()
{

  Context c;
  Context *context = &c;

  _value = 0.0000001;
  assert(asBool(context) == true);

  _value = 1;
  assert(asBool(context) == true);

  _value = 1.89434;
  assert(asBool(context) == true);

  _value = -12.12;
  assert(asBool(context) == true);
  

}

void DataItemNumeric::testAsString()
{

  Context c;
  Context *context = &c;

  _value = 1.234567890123456781;
  delete _cachedString;
  _cachedString = NULL;
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("1.23456789012346"))==0);
  _value = 0.1234567899;
  delete _cachedString;
  _cachedString = NULL;
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("0.1234567899"))==0);  
  _value = 0.0000000001;
  delete _cachedString;
  _cachedString = NULL;
#ifdef WIN32
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("1e-010"))==0);
#else
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("1e-10"))==0);
#endif

  _value = 1;
  delete _cachedString;
  _cachedString = NULL;
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("1"))==0);
  _value = 1.89434;
  delete _cachedString;
  _cachedString = NULL;
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("1.89434"))==0);
  _value = -12.12;
  delete _cachedString;
  _cachedString = NULL;
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("-12.12"))==0);
  _value = 100000000;
  delete _cachedString;
  _cachedString = NULL;
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("100000000"))==0);

}

void DataItemNumeric::testAsInt() {


  Context c;
  Context *context = &c;

  _value = 56.45;
  assert(asInt(context) == 56);

  _value = 56.55;
  assert(asInt(context) == 57);

  _value = 3;
  assert(asInt(context) == 3);


}

void DataItemNumeric::testAsNum()
{

  Context c;
  Context *context = &c;

  _value = 0;
  assert(asNum(context) == 0);

  _value = 1;
  assert(asNum(context) == 1);

  _value = 1.89434;
  assert(asNum(context) == 1.89434);

  _value = -12.12;
  assert(asNum(context) == -12.12);


}

void DataItemNumeric::testAsNodeList()
{

  Context c;
  Context *context = &c;

  _value = 0;
  bool caught = false;
  try {
    asNodeList(context, false);
  }
  catch(const DataItemException&) {
    caught = true;
  }


}

void DataItemNumeric::testEquals()
{

  Context c;
  Context *context = &c;

  _value = 0;
  auto_ptr<DataItem> boolfalse(new DataItemBool(false));
  assert(isEqual(boolfalse.get(), context));

   _value = 1;
  auto_ptr<DataItem> booltrue(new DataItemBool(true));
  assert(isEqual(booltrue.get(), context));

   _value = -54.83;
  assert(isEqual(booltrue.get(), context));

   _value = 6487;
  assert(!isEqual(boolfalse.get(), context));

  _value = 20.764;
  auto_ptr<DataItem> numtwenty(new DataItemNumeric(20.764));
  assert(isEqual(numtwenty.get(), context));

  _value = 54;
  auto_ptr<DataItem> numseventy(new DataItemNumeric(72));
  assert(!isEqual(numseventy.get(), context));

  _value = 20.764;
  auto_ptr<DataItem> stringtwenty(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("20.764")));
  assert(isEqual(stringtwenty.get(), context));

  _value = 54;
  auto_ptr<DataItem> stringseventy(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("72")));
  assert(!isEqual(stringseventy.get(), context));

  _value = 764;
  auto_ptr<DataItem> stringtest(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("this is a test")));
  assert(!isEqual(stringtest.get(), context));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr1 = data->createAttribute( X("one") );
  attr1->setNodeValue(X("jim"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr2 = data->createAttribute( X("two") );
  attr2->setNodeValue(X("76.63"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr3 = data->createAttribute( X("three") );
  attr3->setNodeValue(X("54.4"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr4 = data->createAttribute( X("four") );
  attr4->setNodeValue(X("fred"));

  DeadNodeList *vec1= new DeadNodeList();
  DeadNodeList *vec2= new DeadNodeList();
  vec1->push_back(attr1);
  vec1->push_back(attr2);
  vec2->push_back(attr3);
  vec2->push_back(attr4);
  auto_ptr<DataItem> nodejim(new DataItemNodeList(vec1));
  auto_ptr<DataItem> nodefred(new DataItemNodeList(vec2));

  _value = 76.63;
  assert(isEqual(nodejim.get(), context));

  _value = 54.3;
  assert(!isEqual(nodefred.get(), context));


  /*
    0==false
    1==true
    -54.83==true
    6487!=false

    20.764==20.764
    54!=72

    20.764=="20.764"
    54!="72"
    764!="this is a test"

    76.63==("jim", "76.63")
    54.3!=("54.4", "fred")
   */

}

void DataItemNumeric::testLessThan()
{

  Context c;
  Context *context = &c;

  _value = 1.23;
  auto_ptr<DataItemNodeList> nlist(new DataItemNodeList(new DeadNodeList));
  DataItem *nodeItem = nlist.get();
  assert(isLessThan(nodeItem, context) == false);

  auto_ptr<DataItemNumeric> nNum(new DataItemNumeric(1.23));
  DataItem *numericItem = nNum.get();
  assert(isLessThan(numericItem, context) == false);

  auto_ptr<DataItemNumeric> nNum2(new DataItemNumeric(0.23));
  DataItem *numericItem2 = nNum2.get();
  assert(isLessThan(numericItem2, context) == false);

  auto_ptr<DataItemNumeric> nNum3(new DataItemNumeric(2.23));
  DataItem *numericItem3 = nNum3.get();
  assert(isLessThan(numericItem3, context) == true);

}

void DataItemNumeric::testGreaterThan()
{

  Context c;
  Context *context = &c;

  _value = 1.23;
  auto_ptr<DataItemNodeList> nlist(new DataItemNodeList(new DeadNodeList));
  DataItem *nodeItem = nlist.get();
  assert(isGreaterThan(nodeItem, context) == false);

  auto_ptr<DataItemNumeric> nNum(new DataItemNumeric(1.23));
  DataItem *numericItem = nNum.get();
  assert(isGreaterThan(numericItem, context) == false);

  auto_ptr<DataItemNumeric> nNum2(new DataItemNumeric(0.23));
  DataItem *numericItem2 = nNum2.get();
  assert(isGreaterThan(numericItem2, context) == true);

  auto_ptr<DataItemNumeric> nNum3(new DataItemNumeric(2.23));
  DataItem *numericItem3 = nNum3.get();
  assert(isGreaterThan(numericItem3, context) == false);

}

#endif

