/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "DataItemString.hpp"
#include "../DOMutils/v2Utils.hpp"
#include "DataItem.hpp"
#include "DataItemFunction.hpp"
#include "../exceptions/DataItemException.hpp"
#include "../defs/NumUtils.hpp"
#include <assert.h>
#include <memory>
#include <xercesc/util/XMLUniDefs.hpp>
#include <xercesc/util/Janitor.hpp>

#ifdef TEST
#include "DataItemNodeList.hpp"
#include "DataItemNumeric.hpp"
#include "DataItemBool.hpp"
#endif

DataItemString::DataItemString()
{
  XMLCh* temp = new XMLCh[1];
  *temp = XERCES_CPP_NAMESPACE_QUALIFIER chNull;
  _name = temp;
  setType(DataItem::STRINGLITERAL);
}

DataItemString::DataItemString(const XMLCh *n)
{
  _name = n;
  setType(DataItem::STRINGLITERAL);
}

DataItemString::DataItemString(const DataItemString &rhs)
   : DataItem(rhs), _name(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(rhs._name))
{
}

DataItemString::~DataItemString()
{
   delete[] (XMLCh *)_name;
   _name=NULL;
}

DataItem *DataItemString::asSimpleDataItem(const Context *context) const
{
  return new DataItemString(*this);
}

bool DataItemString::asBool(const Context *context) const
{
  if(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(_name) == 0){
    return false;
  }
  else{
    return true;
  }
}

const XMLCh *DataItemString::asString(const Context *context) const
{
  return _name;
}

/*note this method uses atod so returns a number if there are numerals
and non numerals -- it returns 0 if it cannot convert*/
XSFloat DataItemString::asNum(const Context *context) const
{
  return NumUtils::stringAsDouble(_name);
}

DeadNodeList *DataItemString::asNodeList(const Context *context, const bool singleNode) const
{
  throw DataItemException("DataItemString::asNodeList",
			  "Tried to convert from a string to a nodelist");

  return new DeadNodeList();
}

bool DataItemString::isEqual(const DataItem *other,
			     const Context *context) const
{
  DataItem::whichType otherType = other->getType();

  bool equal = false;

  switch(otherType){
  case  DataItem::STRINGLITERAL:
    // if the strings are identical or the numerals represented by the
    // strings are identical
    equal = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),other->asString(context))==0;
    break;
  case  DataItem::NUMERICLITERAL: {
    equal = NumUtils::isEqual(asNum(context), other->asNum(context));
    break;
  }
  case  DataItem::NODELIST:
  case  DataItem::NAVIGATION: {
    auto_ptr<DeadNodeList> list(other->asNodeList(context, false));
    const XMLCh *myValue = asString(context);

    if(list->getLength()==0 && (myValue==NULL || myValue[0]==0)) {

      return true;
    }
    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){
      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      equal = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(otherString.get(),myValue)==0;
      if(equal) {
	break;
      }
    }
    break;
  }
  case  DataItem::DATABOOLEAN:
    equal = (asBool(context) == other->asBool(context));
    break;
  case  DataItem::VARIABLE:
  case  DataItem::FUNCTION: {
    auto_ptr<DataItem> simpleData
       = static_cast< auto_ptr<DataItem> >(other->asSimpleDataItem(context));
    equal = isEqual(simpleData.get(), context);
    break;
  }
  default:
    assert(false);
    throw DataItemException("DataItemString::isEqual", "enum case not dealt with");
  }

  return equal;
}

bool DataItemString::isLessThan(const DataItem *other,
				const Context *context) const
{
  DataItem::whichType otherType = other->getType();

  bool lessthan = false;

  switch(otherType){
  case  DataItem::STRINGLITERAL:
  case  DataItem::NUMERICLITERAL:
  case  DataItem::DATABOOLEAN:
    lessthan = NumUtils::isLessThan(asNum(context), other->asNum(context));
    break;
  case  DataItem::NODELIST:
  case  DataItem::NAVIGATION: {
    auto_ptr<DeadNodeList> list(other->asNodeList(context, false));

    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){
      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      XSFloat answer = NumUtils::stringAsDouble(otherString.get());
      if(NumUtils::isLessThan(asNum(context),answer)) {
	lessthan = true;
	break;
      }
    }
    break;
  }
  case DataItem::VARIABLE:
  case DataItem::FUNCTION: {
    auto_ptr<DataItem> simpleData
       = static_cast< auto_ptr<DataItem> >(other->asSimpleDataItem(context));
    lessthan = isLessThan(simpleData.get(), context);
    break;
  }
  default:
    assert(false);
    throw DataItemException("DataItemString::isLessThan", "enum case not dealt with");
  }

  return lessthan;
}

bool DataItemString::isGreaterThan(const DataItem *other,
				   const Context *context) const
{
  DataItem::whichType otherType = other->getType();

  bool greaterthan = false;

  switch(otherType){
  case  DataItem::STRINGLITERAL:
  case  DataItem::NUMERICLITERAL:
  case  DataItem::DATABOOLEAN:
    greaterthan = NumUtils::isGreaterThan(asNum(context), other->asNum(context));
    break;
  case  DataItem::NODELIST:
  case  DataItem::NAVIGATION: {
    auto_ptr<DeadNodeList> list(other->asNodeList(context, false));

    for(DeadNodeList::const_iterator i = list->begin();
	i != list->end(); ++i){
      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
      XSFloat answer = NumUtils::stringAsDouble(otherString.get());
      if(NumUtils::isGreaterThan(asNum(context),answer)) {
	greaterthan = true;
	break;
      }
    }
    break;
  }
  case  DataItem::VARIABLE:
  case  DataItem::FUNCTION: {
    auto_ptr<DataItem> simpleData
       = static_cast< auto_ptr<DataItem> >(other->asSimpleDataItem(context));
    greaterthan = isGreaterThan(simpleData.get(), context);
    break;
  }
  default:
    assert(false);
    throw DataItemException("DataItemString::isGreaterThan", "enum case not dealt with");
  }

  return greaterthan;
}

#ifdef TEST
// Test functions

#include "../testharness/TestFunction.hpp"
#include "../DOMutils/XStr.hpp"

void DataItemString::setUp()
{

  // Nothing to do

}

DataItemString::DataItemString(std::string name)
{
  _name = NULL;

  setType(DataItem::NUMERICLITERAL);
}

void DataItemString::testAsBool()
{

  Context c;
  Context *context = &c;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("0");
  assert(asBool(context) == true);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("1");
  assert(asBool(context) == true);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("12.76648");
  assert(asBool(context) == true);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("-78.45");
  assert(asBool(context) == true);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("this is a test");
  assert(asBool(context) == true);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
  assert(asBool(context) == false);
  delete [] _name;
  _name= 0;


}

void DataItemString::testAsString()
{

  Context c;
  Context *context = &c;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("0");
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("0"))==0);
  delete [] _name;
  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("1");
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("1"))==0);
  delete [] _name;
  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("12.76648");
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("12.76648"))==0);
  delete [] _name;
  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("-78.45");
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("-78.45"))==0);
  delete [] _name;
  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("this is a test");
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X("this is a test"))==0);
  delete [] _name;
  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(asString(context),X(""))==0);
  delete [] _name;
  _name= 0;
}

void DataItemString::testAsInt() {


  Context c;
  Context *context = &c;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("56.45");
  assert(asInt(context) == 56);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("56.55");
  assert(asInt(context) == 57);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("3");
  assert(asInt(context) == 3);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("ssd20");
  assert(asInt(context) == 0);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("sd20.2df");
  assert(asInt(context) == 0);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("34.2sd20.2df");
  assert(asInt(context) == 34);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("-89.6453sd20.2df");
  assert(asInt(context) == -90);
  delete [] _name;
  _name= 0;


}

void DataItemString::testAsNum()
{

  Context c;
  Context *context = &c;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("0");
  assert(asNum(context) == 0);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("1");
  assert(asNum(context) == 1);
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("12.76648");
  assert(NumUtils::isEqual(asNum(context), 12.76648L));
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("-78.45");
  assert(NumUtils::isEqual(asNum(context), -78.45L));
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("54.this is a test");
  assert(NumUtils::isNaN(asNum(context))); //NaN
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("this is a test26.2");
  assert(NumUtils::isNaN(asNum(context))); //NaN
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("this is 56 a test");
  assert(NumUtils::isNaN(asNum(context))); //NaN
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("76.4this is 7a test26.2");
  assert(NumUtils::isNaN(asNum(context))); //NaN
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("this is a test");
  assert(NumUtils::isNaN(asNum(context))); //NaN
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
  assert(NumUtils::isNaN(asNum(context))); //NaN
  delete [] _name;
  _name= 0;


}

void DataItemString::testAsNodeList()
{

  Context c;
  Context *context = &c;

  bool caught = false;
  try {
    asNodeList(context, false);
  }
  catch(const DataItemException&) {
    caught = true;
  }
  assert(caught);


}

void DataItemString::testEquals()
{

  Context c;
  Context *context = &c;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
  auto_ptr<DataItem> boolfalse(new DataItemBool(false));
  assert(isEqual(boolfalse.get(), context));

  auto_ptr<DataItem> booltrue(new DataItemBool(true));
  assert(!isEqual(booltrue.get(), context));
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("test string");
  assert(isEqual(booltrue.get(), context));
  assert(!isEqual(boolfalse.get(), context));
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("20.764");
  auto_ptr<DataItem> numtwenty(new DataItemNumeric(20.764));
  assert(isEqual(numtwenty.get(), context));
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("72");
  auto_ptr<DataItem> numfifty(new DataItemNumeric(54));
  assert(!isEqual(numfifty.get(), context));
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("this is a test");
  auto_ptr<DataItem> numseven(new DataItemNumeric(764));
  assert(!isEqual(numseven.get(), context));
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frank fim fred");
  auto_ptr<DataItem> stringfrankjimfred(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frank jim fred")));
  assert(!isEqual(stringfrankjimfred.get(), context));
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("bob bob bob");
  assert(!isEqual(stringfrankjimfred.get(), context));
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frank jim fred mable");
  assert(!isEqual(stringfrankjimfred.get(), context));
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
  auto_ptr<DataItem> stringnull(new DataItemNodeList(new DeadNodeList));
  assert(isEqual(stringnull.get(), context));
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("bob bob bob");
  assert(!isEqual(stringnull.get(), context));
  delete [] _name;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr1 = data->createAttribute( X("one") );
  attr1->setNodeValue(X("bob bob bob"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr2 = data->createAttribute( X("two") );
  attr2->setNodeValue(X("frank jim fred"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr3 = data->createAttribute( X("three") );
  attr3->setNodeValue(X("frank jim fred mable"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attr4 = data->createAttribute( X("four") );
  attr4->setNodeValue(X(""));

  DeadNodeList *vec1= new DeadNodeList();
  DeadNodeList *vec2= new DeadNodeList();
  vec1->push_back(attr1);
  vec1->push_back(attr2);
  vec2->push_back(attr3);
  vec2->push_back(attr4);
  auto_ptr<DataItem> nodebob(new DataItemNodeList(vec1));
  auto_ptr<DataItem> nodefrank(new DataItemNodeList(vec2));

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frank jim fred");
  assert(isEqual(nodebob.get(), context));
  delete [] _name;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
  assert(!isEqual(nodebob.get(), context));
  assert(isEqual(nodefrank.get(), context));
  delete [] _name;
  _name= 0;



  /*
    ""==false
    "test string"==true
    ""!=true
    "test string"!=false

    "20.764"==20.764
    "72"!=54
    "this is a test"!=764

    "frank jim fred"=="frank jim fred"
    "bob bob bob"!="frank jim fred"
    "frank jim fred mable"!="frank jim fred"
    ""==""
    "bob bob bob"!=""

    "frank jim fred"==("bob bob bob", "frank jim fred")
    ""!=("bob bob bob", "frank jim fred")
    ""==("frank jim fred mable", "")
   */

}

void DataItemString::testLessThan()
{

  Context c;
  Context *context = &c;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("10");

  auto_ptr<DataItemNodeList> nList(new DataItemNodeList(new DeadNodeList));
  DataItem *nodeItem = nList.get();
  assert(isLessThan(nodeItem, context) == false);

  auto_ptr<DataItemString> nString(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("20")));
  DataItem *stringItem = nString.get();
  assert(isLessThan(stringItem, context) == true);
  delete [] _name;
  _name= 0;

}

void DataItemString::testGreaterThan()
{

  Context c;
  Context *context = &c;

  _name = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("10");

  auto_ptr<DataItemNodeList> nList(new DataItemNodeList(new DeadNodeList));
  DataItem *nodeItem = nList.get();
  assert(isGreaterThan(nodeItem, context) == false);

  auto_ptr<DataItemString> nString(new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("0")));
  DataItem *stringItem = nString.get();
  assert(isGreaterThan(stringItem, context) == true);
  delete [] _name;
  _name= 0;

}

#endif
