/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "DeadNodeList.hpp"
#include "../DOMutils/DOMNodeCompare.hpp"
#include "../exceptions/NodeListException.hpp"

#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMDocumentFragment.hpp>
#include <xercesc/dom/impl/DOMDocumentImpl.hpp>

#include <algorithm>

using namespace std; // needed for STLport if not gcc

DeadNodeList::DeadNodeList()
{
  documentRoot = NULL;
  _documentChangesSet = false;
}

DeadNodeList::DeadNodeList(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *docRoot) : documentRoot(docRoot)
{
  _documentChanges = -666;
  _documentChangesSet = false;
  setDocumentChanges();
}

DeadNodeList::DeadNodeList(XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList *domList)
{
  documentRoot = NULL;

  _documentChangesSet = false;

  size_t l= domList->getLength();
  _vector.reserve(l);
  for(unsigned int i = 0; i < l; i++) {
    _vector.push_back(domList->item(i));
  }

  setDocumentChanges();
}

DeadNodeList::~DeadNodeList()
{
  // no-op
}

DeadNodeList::operator XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList *() const
{
   return liveList();
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList *DeadNodeList::liveList() const
{
   if(getLength())
   {
      XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *owner = item(0)->getOwnerDocument();

      XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentFragment *frag = owner->createDocumentFragment();

      vector<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *>::const_iterator end = _vector.end();
      for(vector<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *>::const_iterator i = _vector.begin();
          i != end; ++i) {

         if((*i)->getNodeType() != XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE) {
            if((*i)->getOwnerDocument() != owner) {
               throw NodeListException("The nodes in the nodelist are not from the same document");
            } else {
               frag->appendChild(*i);
            }
         } else {
            throw NodeListException("The nodelist contains attribute nodes");
         }

      }
      return frag->getChildNodes();
   }
   else
   {
      return NULL;
   }
}

DeadNodeList *DeadNodeList::clone(bool deep) const
{
  DeadNodeList *result = new DeadNodeList(documentRoot);
  result->_vector.reserve(getLength());
  if(deep) {
    for(vector<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *>::const_iterator i = _vector.begin();
	i != _vector.end(); ++i) {
      result->push_back((*i)->cloneNode(true));
    }
  } else {
    for(vector<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *>::const_iterator i = _vector.begin();
	i != _vector.end(); ++i) {
      result->push_back(*i);
    }
  }

  return result;
}

void DeadNodeList::addList(const DeadNodeList *nodeList)
{
  _vector.reserve(getLength()+nodeList->getLength());
  _vector.insert(_vector.end(), nodeList->_vector.begin(),
		 nodeList->_vector.end());

  setDocumentChanges();
}

void DeadNodeList::addList(XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList *domList)
{
  size_t l= domList->getLength();
  _vector.reserve(getLength()+l);
  for(unsigned int i = 0; i < l; i++) {
    _vector.push_back(domList->item(i));
  }

  setDocumentChanges();
}

bool DeadNodeList::hasDocumentChanged(void) const {

   if(_documentChangesSet == false) {
      throw NodeListException("hasDocumentChanged() called on empty DeadNodeList");
   } else {
      if(static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl*>(documentRoot)->changes() != _documentChanges) {
         return true;
      } else {
         return false;
      }//if
   }//if
}//hasDocumentChanged

XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *DeadNodeList::first() const
{
  vector<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *>::const_iterator ret =
    min_element(_vector.begin(), _vector.end());
  return *ret;
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *DeadNodeList::item(unsigned int index) const
{
  return _vector[index];
}

unsigned int DeadNodeList::getLength(void) const
{
  return _vector.size();
}

void DeadNodeList::push_back(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
  if (node->getOwnerDocument() != NULL) {
    node->getOwnerDocument()->getNodeName();
  }

  _vector.push_back(node);
  setDocumentChanges();
}

void DeadNodeList::push_front(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node)
{
  //This is a hack since the redhat 6.2 deque warns with push_front
  _vector.insert(_vector.begin(), node);;
  setDocumentChanges();
}

void DeadNodeList::clear(void)
{
  _vector.clear();
  _documentChangesSet = false;
}

DeadNodeList::iterator DeadNodeList::begin(void)
{
  return _vector.begin();
}

DeadNodeList::iterator DeadNodeList::end(void)
{
  return _vector.end();
}

bool DeadNodeList::empty(void) const
{
  return _vector.empty();
}

DeadNodeList::const_iterator DeadNodeList::begin(void) const
{
  return _vector.begin();
}

DeadNodeList::const_iterator DeadNodeList::end(void) const
{
  return _vector.end();
}

DeadNodeList::reverse_iterator DeadNodeList::rbegin(void)
{
  return _vector.rbegin();
}

DeadNodeList::reverse_iterator DeadNodeList::rend(void)
{
  return _vector.rend();
}

DeadNodeList::const_reverse_iterator DeadNodeList::rbegin(void) const
{
  return _vector.rbegin();
}

DeadNodeList::const_reverse_iterator DeadNodeList::rend(void) const
{
  return _vector.rend();
}

void DeadNodeList::setDocumentChanges(void)
{
   XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *tmpNode = NULL;

   if(_documentChangesSet == false) {
      //We can get the changes value in two ways:

      //1) From the documentRoot node, if its not null
      if(documentRoot!=NULL) {
         _documentChanges = static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl*>(documentRoot)->changes();
         _documentChangesSet = true;
      }
      else {
         //2) If the list isn't empty, then use the first node
         if(empty() == false) {
            _documentChangesSet = true;
            tmpNode = _vector.front();

            if(tmpNode->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE) {
               documentRoot = (XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *)tmpNode;
            } else {
               documentRoot = tmpNode->getOwnerDocument();
            }//if

            XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl* di= static_cast<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentImpl*>(documentRoot);

            _documentChanges = di->changes();
         }//if
      }//if
   }//if
}//setDocumentChanges

//void DeadNodeList::removeDuplicates()
//{
//	std::sort(begin(), end());
//	DeadNodeList::iterator i= std::unique(begin(), end());
//	_vector.resize(i-begin());
//}

void DeadNodeList::removeDuplicates(bool ordered)
{
  if(ordered) {
    std::vector<XERCES_CPP_NAMESPACE_QUALIFIER DOMNode*> newvector;

    DeadNodeList::const_iterator firstIter, secondIter, end, beforeEnd;

    end = _vector.end();
    beforeEnd = end - 1;

    for(secondIter = begin(); secondIter != end; ++secondIter) {
      bool toAdd = true;
      if(secondIter != beforeEnd) {
        for(firstIter = secondIter + 1; firstIter != end; ++firstIter) {
          if((*firstIter) == (*secondIter)) {
            toAdd = false;
            break;
          }
        }
      }

      if(toAdd == true) {
        newvector.push_back(*secondIter);
      }
    }
    _vector= newvector;
  } else {
	std::sort(begin(), end());
	DeadNodeList::iterator i= std::unique(begin(), end());
	_vector.resize(i-begin());
  }
}
