/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */
#include "NumUtils.hpp"
#include <strstream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <cmath>
#include <xercesc/util/XMLUniDefs.hpp>

#ifdef _MSC_VER
#define snprintf _snprintf
#endif

#ifdef HAVE_IEEEFP_H	//to compile under solaris..
#include <ieeefp.h>
#endif

#define epsilon 2.2204460492503131e-016

// Some portability tricks from Xalan
static const XSFloat s_NaN = sqrt(-2.0);
static const XSFloat s_positiveInfinity = HUGE_VAL;
static const XSFloat s_negativeInfinity = -s_positiveInfinity;
static const XSFloat s_positiveZero = 0.0;
static const XSFloat s_negativeZero = -s_positiveZero;

bool NumUtils::isEqual(XSFloat x, XSFloat y)
{
  if(isNaN(x) || isNaN(y) ||
     (isNegative(x) && !isNegative(y)) ||
     (!isNegative(x) && isNegative(y)) ||
     (isInfinite(x) && !isInfinite(y)) ||
     (!isInfinite(x) && isInfinite(y))) {
    return false;
  }
  // Different algorithm for proximity to 0.
  if(fabs(x) < 1 || fabs(y) < 1) {
    return fabs(x - y) <= epsilon;
  } else {
    return fabs(x - y) <= epsilon * fabs((x + y) / 2);
  }
}

bool NumUtils::isLessThan(XSFloat x, XSFloat y)
{
  if(isNaN(x) || isNaN(y)) {
    return false;
  }
  // Different algorithm for proximity to 0.
  if(fabs(x) < 1 && fabs(y) < 1) {
    return (y - x) > epsilon;
  } else {
    return (y - x) > epsilon * fabs((x + y) / 2);
  }
}

bool NumUtils::isGreaterThan(XSFloat x, XSFloat y)
{
  if(isNaN(x) || isNaN(y)) {
    return false;
  }
  // Different algorithm for proximity to 0.
  if(fabs(x) < 1 && fabs(y) < 1) {
    return (x - y) > epsilon;
  } else {
    return (x - y) > epsilon * fabs((x + y) / 2);
  }
}



bool NumUtils::isNaN(XSFloat n)
{
	return memcmp(&n, &s_NaN, sizeof (XSFloat)) == 0;
}

bool NumUtils::isInfinite(XSFloat n)
{
	return (memcmp(&n, &s_positiveInfinity, sizeof (XSFloat)) == 0) ||
	    (memcmp(&n, &s_negativeInfinity, sizeof (XSFloat)) == 0);
}

bool NumUtils::isNegative(XSFloat n)
{
	return (n < 0.0L) ||
	    (memcmp((char*)&n, (char*)&s_negativeZero, sizeof(XSFloat)) == 0);
}

XMLCh *NumUtils::doubleAsString(XSFloat d)
{
  XMLCh *result = NULL;

  if(NumUtils::isNaN(d)) {
    result = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("NaN");
  } else if(NumUtils::isInfinite(d) && !NumUtils::isNegative(d)) {
    result = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("Infinity");
  } else if(NumUtils::isInfinite(d) && NumUtils::isNegative(d)) {
    result = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("-Infinity");
  } else {
    strstream number;

    /* "precision" sets the fractional part to the maximum possible. */
    number.precision(15);		//XXX: This might be 16..

    number << d << ends;

    result = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(number.str());

    number.freeze(0);

    // If "-0" then "0"
    if(result[0]==XERCES_CPP_NAMESPACE_QUALIFIER chDash && result[1]==XERCES_CPP_NAMESPACE_QUALIFIER chDigit_0 && result[2]==XERCES_CPP_NAMESPACE_QUALIFIER chNull ) {
      result[0]=XERCES_CPP_NAMESPACE_QUALIFIER chDigit_0;
      result[1]=XERCES_CPP_NAMESPACE_QUALIFIER chNull;
    }
  }

  return result;
}

XMLCh *NumUtils::uIntAsString(unsigned int ui)
{
  char n[256];
  snprintf( n, 256, "%u", ui );
  return XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(n);
}

XMLCh *NumUtils::intAsString(unsigned int i)
{
  char n[256];
  snprintf( n, 256, "%i", i );
  return XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(n);
}

XSFloat NumUtils::stringAsDouble(const XMLCh *s)
{
  return stringAsDouble(s, XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(s));
}

int NumUtils::stringAsInt(const XMLCh *s)
{
  return (int)stringAsDouble(s);
}

XSFloat NumUtils::stringAsDouble(const XMLCh *s, const unsigned int length)
{
	if(s == NULL) {
		return s_NaN;
	}//if

	// State variables etc.
	XSFloat tmpVal = 0, sign = 1, pointFactor = 1, base = 0, exponent;
	bool gotBase = false;
	bool gotPoint = false;
	bool gotSign = false;
	bool gotDigit = false;
	bool stop = false;
	bool munchWS = true;
	unsigned int pos = 0;
	XMLCh tmpChar;

	while(!stop && pos < length) {
		tmpChar = s[pos];
		pos++;

		switch(tmpChar) {/*{{{*/

			case L'+': {
						   if(gotSign) {
							   stop = true;
						   } else {
							   sign = 1;
							   gotSign = true;
							   break;
						   }
					   }

			case L'-': {
						   if(gotSign) {
							   stop = true;
						   } else {
							   sign = -1;
							   gotSign = true;
							   break;
						   }
					   }

			case 0x0045:
			case 0x0065: {
							 if(gotBase) {
								 stop = true;
							 } else {
								 //exponent 'e' or 'E'
								 // so store it in base, and reset to read the exponent
								 base = sign * (tmpVal / pointFactor);
								 tmpVal = 0;
								 sign = 1;
								 pointFactor = 1;
								 gotPoint = false;
								 gotSign = false;

								 gotBase = true;
							 }
							 break;
						 }

						 //This is '.'
			case 0x002e: {
							 if(gotPoint) {
								 stop = true;
							 } else {
								 gotPoint = true;
								 pointFactor = 1;
							 }
							 break;
						 }

						 /* All the numerals defined by XML standard */
			case 0x0030:
			case 0x0031:
			case 0x0032:
			case 0x0033:
			case 0x0034:
			case 0x0035:
			case 0x0036:
			case 0x0037:
			case 0x0038:
			case 0x0039: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0030);
							 break;
						 }

			case 0x0660:
			case 0x0661:
			case 0x0662:
			case 0x0663:
			case 0x0664:
			case 0x0665:
			case 0x0666:
			case 0x0667:
			case 0x0668:
			case 0x0669: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0660);
							 break;
						 }

			case 0x06f0:
			case 0x06f1:
			case 0x06f2:
			case 0x06f3:
			case 0x06f4:
			case 0x06f5:
			case 0x06f6:
			case 0x06f7:
			case 0x06f8:
			case 0x06f9: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x06f0);
							 break;
						 }

			case 0x0966:
			case 0x0967:
			case 0x0968:
			case 0x0969:
			case 0x096a:
			case 0x096b:
			case 0x096c:
			case 0x096d:
			case 0x096e:
			case 0x096f: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0966);
							 break;
						 }

			case 0x09e6:
			case 0x09e7:
			case 0x09e8:
			case 0x09e9:
			case 0x09ea:
			case 0x09eb:
			case 0x09ec:
			case 0x09ed:
			case 0x09ee:
			case 0x09ef: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x09e6);
							 break;
						 }

			case 0x0a66:
			case 0x0a67:
			case 0x0a68:
			case 0x0a69:
			case 0x0a6a:
			case 0x0a6b:
			case 0x0a6c:
			case 0x0a6d:
			case 0x0a6e:
			case 0x0a6f: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0a66);
							 break;
						 }

			case 0x0ae6:
			case 0x0ae7:
			case 0x0ae8:
			case 0x0ae9:
			case 0x0aea:
			case 0x0aeb:
			case 0x0aec:
			case 0x0aed:
			case 0x0aee:
			case 0x0aef: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0ae6);
							 break;
						 }

			case 0x0b66:
			case 0x0b67:
			case 0x0b68:
			case 0x0b69:
			case 0x0b6a:
			case 0x0b6b:
			case 0x0b6c:
			case 0x0b6d:
			case 0x0b6e:
			case 0x0b6f: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0b66);
							 break;
						 }

						 //Note: For this language, there appears to be no zero
						 //case 0x0be6
			case 0x0be7:
			case 0x0be8:
			case 0x0be9:
			case 0x0bea:
			case 0x0beb:
			case 0x0bec:
			case 0x0bed:
			case 0x0bee:
			case 0x0bef: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0be6);
							 break;
						 }

			case 0x0c66:
			case 0x0c67:
			case 0x0c68:
			case 0x0c69:
			case 0x0c6a:
			case 0x0c6b:
			case 0x0c6c:
			case 0x0c6d:
			case 0x0c6e:
			case 0x0c6f: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0c66);
							 break;
						 }

			case 0x0ce6:
			case 0x0ce7:
			case 0x0ce8:
			case 0x0ce9:
			case 0x0cea:
			case 0x0ceb:
			case 0x0cec:
			case 0x0ced:
			case 0x0cee:
			case 0x0cef: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0ce6);
							 break;
						 }

			case 0x0d66:
			case 0x0d67:
			case 0x0d68:
			case 0x0d69:
			case 0x0d6a:
			case 0x0d6b:
			case 0x0d6c:
			case 0x0d6d:
			case 0x0d6e:
			case 0x0d6f: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0d66);
							 break;
						 }

			case 0x0e50:
			case 0x0e51:
			case 0x0e52:
			case 0x0e53:
			case 0x0e54:
			case 0x0e55:
			case 0x0e56:
			case 0x0e57:
			case 0x0e58:
			case 0x0e59: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0e50);
							 break;
						 }

			case 0x0ed0:
			case 0x0ed1:
			case 0x0ed2:
			case 0x0ed3:
			case 0x0ed4:
			case 0x0ed5:
			case 0x0ed6:
			case 0x0ed7:
			case 0x0ed8:
			case 0x0ed9: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0ed0);
							 break;
						 }

			case 0x0f20:
			case 0x0f21:
			case 0x0f22:
			case 0x0f23:
			case 0x0f24:
			case 0x0f25:
			case 0x0f26:
			case 0x0f27:
			case 0x0f28:
			case 0x0f29: {
							 gotDigit = true;
							 if(gotPoint) {
								 pointFactor *= 10;
							 }
							 tmpVal *= 10;
							 tmpVal += static_cast<XSFloat>(tmpChar - 0x0f20);
							 break;
						 }

						 // whitespace at start or end of string...
			case 0x0020:
			case 0x0009:
			case 0x000d:
			case 0x000a: {
							 bool endOfWS = false;
							 while(!endOfWS && pos < length) {
								 tmpChar = s[pos];
								 pos++;
								 switch(tmpChar) {
									 case 0x0020:
									 case 0x0009:
									 case 0x000d:
									 case 0x000a: {
													  break;
												  }
									 default: {
												  endOfWS = true;
												  pos--;
												  if(munchWS) {
													  //end of leading whitespace
													  munchWS = false;
												  } else {
													  //trailing whitespace is followed by other characters - so return NaN.
													  stop = true;
												  }
											  }
								 }//switch
							 }//while
							 break;
						 }

			default:
						 stop = true;
						 break;

		}//switch
		/*}}}*/

	}//while

	if(!gotDigit || stop) {
		return s_NaN;
	}

	if(gotBase) {
		exponent = sign * (tmpVal / pointFactor);
	} else {
		base = sign * (tmpVal / pointFactor);
		exponent = 0;
	}

	return base * pow((float)10, (float)exponent);
}

#ifdef TEST
// Test functions
#include "../DOMutils/XStr.hpp"

NumUtils::NumUtils(string name)
  : TestCase(name)
{


}


void NumUtils::setUp(void)
{

  // Nothing to do

}

Test* NumUtils::suite(void)
{

  TestSuite* testSuite = new TestSuite("NumUtils");

  testSuite->addTest(new TestCaller<NumUtils> ("doubleAsString",
					      &NumUtils::testDoubleAsString));
  testSuite->addTest(new TestCaller<NumUtils> ("stringAsDouble",
					      &NumUtils::testStringAsDouble));


  return testSuite;
}

void NumUtils::testDoubleAsString(void)
{

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(doubleAsString(0),X("0"))==0);  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(doubleAsString(1),X("1"))==0);  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(doubleAsString(1.6745),X("1.6745"))==0);  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(doubleAsString(-12.67),X("-12.67"))==0);
}

void NumUtils::testStringAsDouble(void)
{

  assert(fabs(stringAsDouble(X("0")) - 0) < (epsilon));  assert(fabs(stringAsDouble(X("1")) - 1) < (epsilon));  assert(fabs(stringAsDouble(X("12.67")) - 12.67) < (epsilon) * 12.67);  assert(fabs(stringAsDouble(X("-87.53")) - -87.53) < (epsilon) * 87.53);  assert(isNaN(stringAsDouble(X("54.this is a"))));//NaN  assert(isNaN(stringAsDouble(X("this is a test26.2"))));//NaN  assert(isNaN(stringAsDouble(X("this is 56 a test"))));//NaN  assert(isNaN(stringAsDouble(X("76.4this is 7a test26.2"))));//NaN  assert(isNaN(stringAsDouble(X("this is a test"))));//NaN  assert(isNaN(stringAsDouble(X(""))));//NaN
}

#endif // TEST



/* vim600: set foldmethod=marker : */
