/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionId.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../context/Context.hpp"
#include "../exceptions/FunctionException.hpp"
#include <assert.h>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/util/XMLUniDefs.hpp>
#include <xercesc/util/Janitor.hpp>
#include <memory>

FunctionId::FunctionId(const vector<const DataItem*> &args)
  : XPathFunction(1, 1, args)
{
}

DataItem *FunctionId::execute(const Context *context) const
{
  if(context->getCurrentContext()!=NULL && (context->getCurrentContext()->getNodeType()==XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE || context->getCurrentContext()->getOwnerDocument()!=NULL)) {

    // evaluate any functions/navigations etc.
    auto_ptr<DataItem> arg
       = static_cast< auto_ptr<DataItem> >(getDataItemArg(0, context)->asSimpleDataItem(context));

    if(arg->getType() == DataItem::NODELIST) {
      //Nodelist case (use strings from all nodes in the list)
      DeadNodeList *result = new DeadNodeList();
      auto_ptr<DeadNodeList> argList(arg->asNodeList(context, false));
      for(DeadNodeList::const_iterator i = argList->begin();
	  i != argList->end(); ++i) {
         XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> otherString(v2Utils::getStringFromNode(*i));
         auto_ptr<DeadNodeList> tempList
            = static_cast< auto_ptr<DeadNodeList> >(findID(otherString.get(),context));
         result->addList(tempList.get());
      }
      return new DataItemNodeList(result);

    } else {
      // Simple string case.
      XERCES_CPP_NAMESPACE_QUALIFIER ArrayJanitor<XMLCh> string(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(arg->asString(context)));
      return new DataItemNodeList(findID(string.get(),context));
    }

  } else {
    throw FunctionException("FunctionId", "Current context has no owner document");
  }
}

DeadNodeList *FunctionId::findID(XMLCh *values,
                                 const Context *context) const
{
  // NB. Writes to 'values' string

  DeadNodeList *result = new DeadNodeList;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc = context->getCurrentContext()->getOwnerDocument();

  if (doc == NULL) {
    // We assume that this is because the CurrentContext /IS/ the DOMDocument
    // We should probably explicitly check this here.
    doc = (XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *)(context->getCurrentContext());
  }

  bool munchWS = true; // munch initial WS
  unsigned int start = 0;

  /* XPath requires this bizarre WS separated splitting of the string, as the
     string can hold many id's. */
  for(unsigned int i = 0; values[i]!=XERCES_CPP_NAMESPACE_QUALIFIER chNull; ++i) {
    switch(values[i]) {
      case XERCES_CPP_NAMESPACE_QUALIFIER chSpace:
      case XERCES_CPP_NAMESPACE_QUALIFIER chHTab:
      case XERCES_CPP_NAMESPACE_QUALIFIER chCR:
      case XERCES_CPP_NAMESPACE_QUALIFIER chLF: {
      if(!munchWS) {
	munchWS = true;

	// Found the end of an ID, so find the element associated with it.
          values[i]=XERCES_CPP_NAMESPACE_QUALIFIER chNull;
          XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *temp = doc->getElementById(values + start);
	if(temp!=NULL) {
	  result->push_back(temp);
	}
      }
      break;
    }
    default: {
      if(munchWS) {
	//found the start of an ID
	start = i;
	munchWS = false;
      }
    }
    }
  }
  if(!munchWS) {
    // Found the end of an ID, so find the element associated with it.
    XERCES_CPP_NAMESPACE_QUALIFIER DOMElement *temp = doc->getElementById(values + start);
    if(temp!=NULL) {
      result->push_back(temp);
    }
  }

  return result;
}

#ifdef TEST
// Test functions
#include <xercesc/parsers/XercesDOMParser.hpp>
#include "../dataItem/DataItemString.hpp"
#include "../simpleVariables/VariableStore.hpp"
#include "../functionAPI/FunctionLookup.hpp"
#include "../testharness/TestFunction.hpp"
#include "../dataItem/DataItemNumeric.hpp"
#include "../DOMutils/XStr.hpp"

void FunctionId::setUp()
{

  // Nothing to do

}

FunctionId::FunctionId( std::string name)
   : XPathFunction(2, 2, vector<const DataItem*>())
{




}

void FunctionId::testExecute()
{


  TestFunction test;

  XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser xmlparser;
  test.initDOMParser(xmlparser);

  XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *script = test.parseXML(xmlparser, strdup("id.data.xml"));
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(test.initDataTree());
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> out(test.initOutputTree());

  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *node = script->getDocumentElement();

  Context c(node, node, new VariableStore, new FunctionLookup, XPathResult::ANY_TYPE);
  Context *context = &c;

  DataItem *test1 = new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("one"));
  _args.push_back(test1);


  auto_ptr<DataItem> result(execute(context));

  assert(result->asNodeList(context, false)->getLength() == 1);
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result->asNodeList(context, false)->item(0)->getNodeName(),X("book"))==0);
  _args.clear();
  DataItem *test2 = new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("frank"));
  _args.push_back(test2);



  result.reset(execute(context));

  assert(result->asNodeList(context, false)->getLength() == 0);

  _args.clear();
  DataItem *test3 = new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("    one  oneOne  oneOneOne"));
  _args.push_back(test3);



  result.reset(execute(context));

  assert(result->asNodeList(context, false)->getLength() == 3);
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result->asNodeList(context, false)->item(0)->getNodeName(),X("book"))==0);  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result->asNodeList(context, false)->item(1)->getNodeName(),X("chapter"))==0);  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result->asNodeList(context, false)->item(2)->getNodeName(),X("section"))==0);
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node1 = script->createTextNode(X("oneOneTwo"));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node2 = script->createTextNode(X("one oneOne"));
  DeadNodeList *nl = new DeadNodeList;
  nl->push_back(node1);
  nl->push_back(node2);
  DataItem *test4 = new DataItemNodeList(nl);
  _args.clear();
  _args.push_back(test4);



  result.reset(execute(context));

  assert(result->asNodeList(context, false)->getLength() == 3);
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result->asNodeList(context, false)->item(0)->getNodeName(),X("section"))==0);  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result->asNodeList(context, false)->item(1)->getNodeName(),X("book"))==0);  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result->asNodeList(context, false)->item(2)->getNodeName(),X("chapter"))==0);
  _args.clear();
  DataItem *test5 = new DataItemNodeList(new DeadNodeList());
  _args.push_back(test5);



  result.reset(execute(context));

  assert(result->asNodeList(context, false)->getLength() == 0);

  DeadNodeList *nl2 = new DeadNodeList;
  DataItem *test6 = new DataItemNodeList(nl2);
  _args.clear();
  _args.push_back(test6);



  result.reset(execute(context));

  assert(result->asNodeList(context, false)->getLength() == 0);

  _args.clear();
}

#endif
