/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionLocalname.hpp"
#include <assert.h>
#include "../exceptions/FunctionException.hpp"
#include "../context/Context.hpp"
#include "../dataItem/DataItemString.hpp"
#include <memory>

FunctionLocalname::FunctionLocalname(const vector<const DataItem*> &args)
  : XPathFunction(0, 1, args)
{
}

DataItem *FunctionLocalname::execute(const Context *context) const
{
  // Use the nodelist
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *node = NULL;

  if(getNumArgs() == 1) {
    auto_ptr<DeadNodeList> nodelist(getNodeListArg(0, context));
    if(nodelist->getLength()) {
      node = nodelist->first();
    }
  } else {
    node = context->getCurrentContext();
  }

  if(node!=NULL) {
    switch(node->getNodeType()){
    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ATTRIBUTE_NODE:
    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ELEMENT_NODE: {
      const XMLCh *result = node->getLocalName();
      if( result == 0 ) {
	result = node->getNodeName();
      }
      return new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(result));
      break;
    }
    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::TEXT_NODE: {
      return new DataItemString();
      break;
    }
    default: {
      assert(false);
      throw(FunctionException("FunctionLocalname", "Unknown node type"));
      break;
    }
    }
    }
  else {
    return new DataItemString();
  }
}

#ifdef TEST
// Test functions

#include "../testharness/TestFunction.hpp"
#include "../simpleVariables/VariableStore.hpp"
#include "../functionAPI/FunctionLookup.hpp"
#include "../DOMutils/XStr.hpp"

void FunctionLocalname::setUp()
{

  // Nothing to do

}

FunctionLocalname::FunctionLocalname( std::string name)
   : XPathFunction(2, 2, vector<const DataItem*>())
{




}

void FunctionLocalname::testExecute()
{
  Context c;
  Context *context = &c;

  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *factory = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(factory->createDocument(0, X("test"), NULL));

  DeadNodeList *nodelist1= new DeadNodeList;
  
  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *elementnode1 = data->createElementNS(X("mavis.com"), X("frank"));
  elementnode1->setPrefix(X("mavis"));
  nodelist1->push_back(elementnode1);
  auto_ptr<DataItem> test1(new DataItemNodeList(nodelist1));
  _args.push_back(test1.get());

  Context context2;
  

  auto_ptr<DataItem> result1(execute(context));
  
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result1->asString(context),X("frank"))==0);
  DeadNodeList *nodelist2= new DeadNodeList;
  _args.clear();
  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement  *elementnode2 = data->createElement(X("frank"));
  nodelist2->push_back(elementnode2);
  auto_ptr<DataItem> test2(new DataItemNodeList(nodelist2));
  _args.push_back(test2.get());


  auto_ptr<DataItem> result2(execute(context));
  
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result2->asString(context),X("frank"))==0); 
  DeadNodeList *nodelist3= new DeadNodeList;
  _args.clear();
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attributenode1 = data->createAttributeNS(X("jim.com"), X("bob"));
  attributenode1->setPrefix(X("jim"));
  nodelist3->push_back(attributenode1);
  auto_ptr<DataItem> test3(new DataItemNodeList(nodelist3));
  _args.push_back(test3.get());


  auto_ptr<DataItem> result3(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result3->asString(context),X("bob"))==0); 
  DeadNodeList *nodelist4= new DeadNodeList;
  _args.clear();
  XERCES_CPP_NAMESPACE_QUALIFIER DOMAttr *attributenode2 = data->createAttribute(X("bob"));
  nodelist4->push_back(attributenode2);
  auto_ptr<DataItem> test4(new DataItemNodeList(nodelist4));
  _args.push_back(test4.get());


  auto_ptr<DataItem> result4(execute(context));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result4->asString(context),X("bob"))==0);
  DeadNodeList *nodelist5= new DeadNodeList;
  _args.clear();
  XERCES_CPP_NAMESPACE_QUALIFIER DOMText *textnode = data->createTextNode(X("charlie"));
  nodelist5->push_back(textnode);
  auto_ptr<DataItem> test5(new DataItemNodeList(nodelist5));
  _args.push_back(test5.get());


  auto_ptr<DataItem> result5(execute(context)); 

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result5->asString(context),X(""))==0); 
  DeadNodeList *nodelist6= new DeadNodeList;
  _args.clear();
  auto_ptr<DataItem> test6(new DataItemNodeList(nodelist6));
  _args.push_back(test6.get());

  auto_ptr<DataItem> result6(execute(context));
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result6->asString(context),X(""))==0);
  _args.clear();

  VariableStore store3;
  FunctionLookup functionLookup3;
  Context context3(elementnode1, NULL, &store3, &functionLookup3, XPathResult::ANY_TYPE);

  auto_ptr<DataItem> result7(execute(&context3));

  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(result7->asString(context),X("frank"))==0);
  /*
    ns:element
    element
    ns:attribute
    attribute
    text
    *current context*
  */

}

#endif
