/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "FunctionUnion.hpp"
#include "../dataItem/DeadNodeList.hpp"
#include "../DOMutils/v2Utils.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include <memory>
#include "../defs/NumUtils.hpp"

FunctionUnion::FunctionUnion(const vector<const DataItem*> &args)
  : XPathFunction(2, 2, args)
{
}

DataItem *FunctionUnion::execute(const Context *context) const
{
  DeadNodeList *firstList = getNodeListArg(0, context);
  auto_ptr<DeadNodeList> secondList(getNodeListArg(1, context));

  //Now let's go through each element in the second list, and if it isn't already
  //in the first list, then add it to the first list
  for(DeadNodeList::const_iterator secondIter = secondList->begin(); secondIter != secondList->end(); secondIter++) {
    bool toAdd = true;
    for(DeadNodeList::iterator firstIter = firstList->begin(); firstIter != firstList->end(); firstIter++) {
      //If we see this element then set toAdd to false, and exit
      if((*firstIter) == (*secondIter)) {
        toAdd = false;
        break;
      }//if   
    }//for

    //Add the element if it isn't already in the list
    if(toAdd == true) {
      firstList->push_back(*secondIter);
    }//if
  }//for

  return new DataItemNodeList(firstList);
}

#ifdef TEST
// Test functions

#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemBool.hpp"
#include "../DOMutils/XStr.hpp"

#include "../testharness/TestFunction.hpp"

void FunctionUnion::setUp()
{

  // Nothing to do

}

FunctionUnion::FunctionUnion(std::string name) : XPathFunction(2, 2, vector<const DataItem*>())
{





}

void FunctionUnion::testExecute()
{


  
  XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation *impl = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation::getImplementation();
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> doc(impl->createDocument(X("http://www.xmlscript.org/2.0"), X("xs:__data__"), NULL));
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *temp;
  
  //Test1: Boundary/Base case of a single element in each list (distinct)
  {
  DeadNodeList *dataItems1 = new DeadNodeList;
  DeadNodeList *dataItems2 = new DeadNodeList;
  temp = doc->createElement(X("one"));
  dataItems1->push_back(temp);
  
  temp = doc->createElement(X("two"));
  dataItems2->push_back(temp);

  auto_ptr<DataItemNodeList> firstList(new DataItemNodeList(dataItems1));
  DataItem *wrapper1 = firstList.get();

  auto_ptr<DataItemNodeList> secondList(new DataItemNodeList(dataItems2));
  DataItem *wrapper2 = secondList.get();

  _args.clear();
  _args.push_back(wrapper1);
  _args.push_back(wrapper2);


 
  Context c;
  Context *context = &c;
  auto_ptr<DataItem> resultdi(execute(context));
  auto_ptr<DeadNodeList> result(resultdi->asNodeList(context, false));
 
  //Verify results
  const XMLCh *node;

  node = result->item(0)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("one"),node)==0);  node = result->item(1)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("two"),node)==0);  assert(result->getLength() == 2);
  }//Test1

  //Test2: Boundary/Base case of a single element in each list (same)
  {
  DeadNodeList *dataItems1 = new DeadNodeList;
  DeadNodeList *dataItems2 = new DeadNodeList;
  temp = doc->createElement(X("one"));
  dataItems1->push_back(temp);
  dataItems2->push_back(temp);

  auto_ptr<DataItemNodeList> firstList(new DataItemNodeList(dataItems1));
  DataItem *wrapper1 = firstList.get();

  auto_ptr<DataItemNodeList> secondList(new DataItemNodeList(dataItems2));
  DataItem *wrapper2 = secondList.get();

  _args.clear();
  _args.push_back(wrapper1);
  _args.push_back(wrapper2);


 
  Context c;
  Context *context = &c;
  auto_ptr<DataItem> resultdi(execute(context));
  auto_ptr<DeadNodeList> result(resultdi->asNodeList(context, false));
 
  //Verify results
  const XMLCh *node;

  node = result->item(0)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("one"),node)==0);  assert(result->getLength() == 1);
  }//Test2

  //Test3: Boundary/Base case of a single element in first list, and two elements in second list (disjoint sets)
  {
  DeadNodeList *dataItems1 = new DeadNodeList;
  DeadNodeList *dataItems2 = new DeadNodeList;
  temp = doc->createElement(X("one"));
  dataItems1->push_back(temp);

  temp = doc->createElement(X("two"));
  dataItems2->push_back(temp);
  temp = doc->createElement(X("three"));
  dataItems2->push_back(temp);

  auto_ptr<DataItemNodeList> firstList(new DataItemNodeList(dataItems1));
  DataItem *wrapper1 = firstList.get();

  auto_ptr<DataItemNodeList> secondList(new DataItemNodeList(dataItems2));
  DataItem *wrapper2 = secondList.get();

  _args.clear();
  _args.push_back(wrapper1);
  _args.push_back(wrapper2);


 
  Context c;
  Context *context = &c;
  auto_ptr<DataItem> resultdi(execute(context));
  auto_ptr<DeadNodeList> result(resultdi->asNodeList(context, false));
 
  //Verify results
  const XMLCh *node;

  node = result->item(0)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("one"),node)==0);  node = result->item(1)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("two"),node)==0);  node = result->item(2)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("three"),node)==0);  assert(result->getLength() == 3);
  }//Test3

  //Test4: Boundary/Base case of a single element in first list, and two elements in second list (not disjoint sets)
  {
  DeadNodeList *dataItems1 = new DeadNodeList;
  DeadNodeList *dataItems2 = new DeadNodeList;
  temp = doc->createElement(X("one"));
  dataItems1->push_back(temp);

  dataItems2->push_back(temp);
  temp = doc->createElement(X("three"));
  dataItems2->push_back(temp);

  auto_ptr<DataItemNodeList> firstList(new DataItemNodeList(dataItems1));
  DataItem *wrapper1 = firstList.get();

  auto_ptr<DataItemNodeList> secondList(new DataItemNodeList(dataItems2));
  DataItem *wrapper2 = secondList.get();

  _args.clear();
  _args.push_back(wrapper1);
  _args.push_back(wrapper2);


 
  Context c;
  Context *context = &c;
  auto_ptr<DataItem> resultdi(execute(context));
  auto_ptr<DeadNodeList> result(resultdi->asNodeList(context, false));
 
  //Verify results
  const XMLCh *node;

  node = result->item(0)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("one"),node)==0);  node = result->item(1)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("three"),node)==0);  assert(result->getLength() == 2);
  }//Test4

  //Test5: Boundary/Base case of two elements in first list, and a single element in second list (disjoint sets)
  {
  DeadNodeList *dataItems1 = new DeadNodeList;
  DeadNodeList *dataItems2 = new DeadNodeList;
  temp = doc->createElement(X("one"));
  dataItems2->push_back(temp);

  temp = doc->createElement(X("two"));
  dataItems1->push_back(temp);
  temp = doc->createElement(X("three"));
  dataItems1->push_back(temp);

  auto_ptr<DataItemNodeList> firstList(new DataItemNodeList(dataItems1));
  DataItem *wrapper1 = firstList.get();

  auto_ptr<DataItemNodeList> secondList(new DataItemNodeList(dataItems2));
  DataItem *wrapper2 = secondList.get();

  _args.clear();
  _args.push_back(wrapper1);
  _args.push_back(wrapper2);


 
  Context c;
  Context *context = &c;
  auto_ptr<DataItem> resultdi(execute(context));
  auto_ptr<DeadNodeList> result(resultdi->asNodeList(context, false));
 
  //Verify results
  const XMLCh *node;

  node = result->item(0)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("two"),node)==0);  node = result->item(1)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("three"),node)==0);  node = result->item(2)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("one"),node)==0);  assert(result->getLength() == 3);
  }//Test5

  //Test6: Boundary/Base case of two elements in first list, and a single element in second list (not disjoint sets)
  {
  DeadNodeList *dataItems1 = new DeadNodeList;
  DeadNodeList *dataItems2 = new DeadNodeList;

  temp = doc->createElement(X("two"));
  dataItems1->push_back(temp);
  temp = doc->createElement(X("three"));
  dataItems1->push_back(temp);

  dataItems2->push_back(temp);

  auto_ptr<DataItemNodeList> firstList(new DataItemNodeList(dataItems1));
  DataItem *wrapper1 = firstList.get();

  auto_ptr<DataItemNodeList> secondList(new DataItemNodeList(dataItems2));
  DataItem *wrapper2 = secondList.get();

  _args.clear();
  _args.push_back(wrapper1);
  _args.push_back(wrapper2);


 
  Context c;
  Context *context = &c;
  auto_ptr<DataItem> resultdi(execute(context));
  auto_ptr<DeadNodeList> result(resultdi->asNodeList(context, false));
 
  //Verify results
  const XMLCh *node;

  node = result->item(0)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("two"),node)==0);  node = result->item(1)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("three"),node)==0);  assert(result->getLength() == 2);
  }//Test6

  //Test7: Boundary/Base case of one list being empty
  {
  DeadNodeList *dataItems1 = new DeadNodeList;
  DeadNodeList *dataItems2 = new DeadNodeList;

  temp = doc->createElement(X("two"));
  dataItems1->push_back(temp);
  temp = doc->createElement(X("three"));
  dataItems1->push_back(temp);

  auto_ptr<DataItemNodeList> firstList(new DataItemNodeList(dataItems1));
  DataItem *wrapper1 = firstList.get();

  auto_ptr<DataItemNodeList> secondList(new DataItemNodeList(dataItems2));
  DataItem *wrapper2 = secondList.get();

  _args.clear();
  _args.push_back(wrapper1);
  _args.push_back(wrapper2);


 
  Context c;
  Context *context = &c;
  auto_ptr<DataItem> resultdi(execute(context));
  auto_ptr<DeadNodeList> result(resultdi->asNodeList(context, false));
 
  //Verify results
  const XMLCh *node;

  node = result->item(0)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("two"),node)==0);  node = result->item(1)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("three"),node)==0);  assert(result->getLength() == 2);
  }//Test7

  //Test8: Boundary/Base case of both lists being empty
  {
  DeadNodeList *dataItems1 = new DeadNodeList;
  DeadNodeList *dataItems2 = new DeadNodeList;

  auto_ptr<DataItemNodeList> firstList(new DataItemNodeList(dataItems1));
  DataItem *wrapper1 = firstList.get();

  auto_ptr<DataItemNodeList> secondList(new DataItemNodeList(dataItems2));
  DataItem *wrapper2 = secondList.get();

  _args.clear();
  _args.push_back(wrapper1);
  _args.push_back(wrapper2);


 
  Context c;
  Context *context = &c;
  auto_ptr<DataItem> resultdi(execute(context));
  auto_ptr<DeadNodeList> result(resultdi->asNodeList(context, false));
 
  //Verify results
  assert(result->getLength() == 0);
  }//Test8

  //Test9: General case example/test
  {
  DeadNodeList *dataItems1 = new DeadNodeList;
  DeadNodeList *dataItems2 = new DeadNodeList;

  temp = doc->createElement(X("two"));
  dataItems1->push_back(temp);
  temp = doc->createElement(X("three"));
  dataItems1->push_back(temp);

  temp = doc->createElement(X("one"));
  dataItems2->push_back(temp);
  temp = doc->createElement(X("four"));
  dataItems2->push_back(temp);

  temp = doc->createElement(X("five"));
  dataItems2->push_back(temp);
  
  auto_ptr<DataItemNodeList> firstList(new DataItemNodeList(dataItems1));
  DataItem *wrapper1 = firstList.get();

  auto_ptr<DataItemNodeList> secondList(new DataItemNodeList(dataItems2));
  DataItem *wrapper2 = secondList.get();

  _args.clear();
  _args.push_back(wrapper1);
  _args.push_back(wrapper2);

  Context c;
  Context *context = &c;
  auto_ptr<DataItem> resultdi(execute(context));
  auto_ptr<DeadNodeList> result(resultdi->asNodeList(context, false));
 
  //Verify results
  const XMLCh *node;

  node = result->item(0)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("two"),node)==0);  node = result->item(1)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("three"),node)==0);  node = result->item(2)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("one"),node)==0);  node = result->item(3)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("four"),node)==0);  node = result->item(4)->getNodeName();
  assert(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::compareString(X("five"),node)==0);  assert(result->getLength() == 5);
  }//Test9

  _args.clear();
}
#endif
