/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "XPathEvaluatorImpl.hpp"

#include "XPathExpressionImpl.hpp"
#include "XPathNSResolverImpl.hpp"

#include <dom3-xpath/XPathExpression.hpp>

#include <dataItem/DataItemBool.hpp>
#include <dataItem/DataItemNodeList.hpp>
#include <dataItem/DataItemNumeric.hpp>
#include <dataItem/DataItemString.hpp>

#include <simpleVariables/VariableStore.hpp>

#include <parser/XPathParserControl.hpp>

#include <functionAPI/FunctionLookup.hpp>
#include <functionAPI/XPathFunctionManifest.hpp>

#include <DOMutils/v2Utils.hpp>

#include <exceptions/DataItemException.hpp>
#include <exceptions/NavigationException.hpp>
#include <exceptions/XPathParseException.hpp>
#include <exceptions/FunctionException.hpp>
#include <exceptions/ContextException.hpp>

#include <xercesc/dom/DOM.hpp>

#define yyFlexLexer xxFlexLexer
#include <lexer/XPathFlexLexer.hpp>
#undef yyFlexLexer

//We don't care about the two unused exception variables
#ifdef WIN32
#pragma warning(disable: 4101)
#endif


//Bison hooks
extern int XPathyyparse(void *);
extern int XPathyydebug;

XPathEvaluatorImpl::XPathEvaluatorImpl()
{
   functionTable = new FunctionLookup();
   XPathFunctionManifest::populate(functionTable);
   variableStore = new VariableStore();
}

XPathEvaluatorImpl::~XPathEvaluatorImpl()
{
   delete functionTable, functionTable=NULL;
   delete variableStore, variableStore=NULL;
}

XPathExpression *XPathEvaluatorImpl::createExpression( const XMLCh *expression,
                                                       const XPathNSResolver *resolver)
   const throw (XPathException, XERCES_CPP_NAMESPACE_QUALIFIER DOMException)
{

   //Generate Context from contextNode
   XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *contextNode = NULL;
   if(resolver!=NULL)
   {
      const XPathNSResolverImpl *resolverImpl = static_cast<const XPathNSResolverImpl*>(resolver);
      if(resolverImpl!=NULL)
      {
         contextNode = resolverImpl->getResolverNode();
      }
   }

   //Lex expression
   LexerDOMStringStream lexerStream(expression);
   xxFlexLexer lexer(&lexerStream, &cout);

   //Parse expression
   XPathParserControl pcontrol;
   pcontrol.lexer = &lexer;
   pcontrol.flook = functionTable;

   XPathyydebug = 0;

   try {
      XPathyyparse((void *) &pcontrol);
   }
   catch(const ContextException &e) {
      throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException(XERCES_CPP_NAMESPACE_QUALIFIER DOMException::NAMESPACE_ERR, XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(e.getError().c_str()));
   }
   catch(const XPathParseException &e)
   {
      string errString;
      errString += "The expression '";
      errString += v2Utils::DSLtranscode(expression);
      errString += "' is an invalid XPath expression";
      throw XPathException(XPathException::INVALID_EXPRESSION_ERR, errString);
   }
   catch(const FunctionException &fe)
   {
      string errString;
      errString += fe.getError();
      throw XPathException(XPathException::INVALID_EXPRESSION_ERR, errString);
   }
   catch(const DSLException &e) {
      string errString;
      errString += "The expression '";
      errString += v2Utils::DSLtranscode(expression);
      errString += "' isn't recognised as a valid XPath expression";
      throw XPathException(XPathException::INVALID_EXPRESSION_ERR, errString);
   }

   return new XPathExpressionImpl(pcontrol.result, resolver, functionTable, variableStore);
}//createExpression

XPathResult *XPathEvaluatorImpl::evaluate(const XMLCh *expression,
                                          XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *contextNode,
                                          const XPathNSResolver *resolver,
                                          const XPathResult::resultType type,
                                          XPathResult *result) const
   throw (XPathException, XERCES_CPP_NAMESPACE_QUALIFIER DOMException)
{
   //Parse expression
   auto_ptr<XPathExpression> parsedExpression
      = static_cast< auto_ptr<XPathExpression> >(createExpression(expression, resolver));

   return parsedExpression->evaluate(contextNode, type, result);
}//evaluate

XPathNSResolver *XPathEvaluatorImpl::createNSResolver(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *nodeResolver) const
{
   return new XPathNSResolverImpl(nodeResolver);
}//createNSResolver
  
void XPathEvaluatorImpl::setGlobalVar(const XMLCh *name, const XMLCh *value)
{
	Context context(NULL, NULL, variableStore, functionTable, XPathResult::ANY_TYPE);
	variableStore->setGlobalVar(name, new DataItemString(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::replicate(value)), &context);
}//setGlobalVar

void XPathEvaluatorImpl::setGlobalVar(const XMLCh *name, bool value)
{
	Context context(NULL, NULL, variableStore, functionTable, XPathResult::ANY_TYPE);
	variableStore->setGlobalVar(name, new DataItemBool(value), &context);
}//setGlobalVar

void XPathEvaluatorImpl::setGlobalVar(const XMLCh *name, double value)
{
	Context context(NULL, NULL, variableStore, functionTable, XPathResult::ANY_TYPE);
	variableStore->setGlobalVar(name, new DataItemNumeric(value), &context);
}//setGlobalVar

void XPathEvaluatorImpl::setGlobalVar(const XMLCh *name, XERCES_CPP_NAMESPACE_QUALIFIER DOMNodeList *value)
{
	Context context(NULL, NULL, variableStore, functionTable, XPathResult::ANY_TYPE);
	variableStore->setGlobalVar(name, new DataItemNodeList(new DeadNodeList(value)), &context);
}//setGlobalVar

void XPathEvaluatorImpl::setGlobalVar(const XMLCh *name, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *value)
{
	Context context(NULL, NULL, variableStore, functionTable, XPathResult::ANY_TYPE);
	variableStore->setGlobalVar(name, new DataItemNodeList(new DeadNodeList(value)), &context);
}//setGlobalVar

