/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "XPathExpressionImpl.hpp"
#include "XPathResultImpl.hpp"
#include "XPathNSResolverImpl.hpp"

#include <context/Context.hpp>
#include <dataItem/DataItemString.hpp>
#include <dataItem/DataItemNumeric.hpp>
#include <dataItem/DataItemBool.hpp>
#include <dataItem/DataItemNodeList.hpp>

#include <exceptions/DataItemException.hpp>
#include <exceptions/NavigationException.hpp>
#include <exceptions/XPathParseException.hpp>
#include <exceptions/FunctionException.hpp>
#include <exceptions/ContextException.hpp>

XPathExpressionImpl::XPathExpressionImpl(DataItem *compiledExpr,
                                         const XPathNSResolver *resolver,
                                         const FunctionLookup *funcTable,
					 VariableStore *varStore)
  : compiledExpression(compiledExpr), nsResolver(resolver), functionTable(funcTable), variableStore(varStore)
{
}//constructor

XPathExpressionImpl::~XPathExpressionImpl()
{
   delete compiledExpression, compiledExpression=NULL;
}//destructor

XPathResult *XPathExpressionImpl::evaluate(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *contextNode,
                                          const XPathResult::resultType type,
                                          XPathResult *result) const
   throw (XPathException, XERCES_CPP_NAMESPACE_QUALIFIER DOMException)
{
	//Generate Context from contextNode - check trigger node correctness
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *resolverNode = NULL;
  if(nsResolver!=NULL)
  {
     const XPathNSResolverImpl *resolverImpl = static_cast<const XPathNSResolverImpl*>(nsResolver);
     if(resolverImpl!=NULL)
     {
        resolverNode = resolverImpl->getResolverNode();
     }
  }

  //Check for illegal resolverNode types
  if(contextNode==NULL) {
    throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException(XERCES_CPP_NAMESPACE_QUALIFIER DOMException::NOT_SUPPORTED_ERR, XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("Context node is null."));
  }

  if(contextNode->getNodeType()!=XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE && contextNode->getOwnerDocument()==NULL) {
    throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException(XERCES_CPP_NAMESPACE_QUALIFIER DOMException::NOT_SUPPORTED_ERR, XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("Context node has null owner document."));
  }

  //More illegal types here?
  switch (contextNode->getNodeType()) {

    case XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::ENTITY_REFERENCE_NODE:
      throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException(XERCES_CPP_NAMESPACE_QUALIFIER DOMException::NOT_SUPPORTED_ERR, XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("Context node of illegal type."));
      break;
  }

  auto_ptr<Context> context = static_cast< auto_ptr<Context> >(new
	Context(contextNode, resolverNode, variableStore,functionTable, type));

  //Evaluate resulting DataItemNav in correct context
  const DataItem *navigation = compiledExpression;

  DataItem *resultItem = NULL;

  //Set the result type to the requested type, may be overridden

  XPathResult::resultType resultType = type;

  try {
    switch (type) {
      case XPathResult::NUMBER_TYPE:
        resultItem = new DataItemNumeric(navigation->asNum(context.get()));
        break;

      case XPathResult::STRING_TYPE:
        resultItem = new DataItemString(navigation->asString(context.get()));
        break;

      case XPathResult::BOOLEAN_TYPE:
        resultItem = new DataItemBool(navigation->asBool(context.get()));
        break;

        //Single node types
      case XPathResult::ANY_UNORDERED_NODE_TYPE:
      case XPathResult::FIRST_ORDERED_NODE_TYPE:
        //Request a single node
        resultItem = new DataItemNodeList(navigation->asNodeList(context.get(), true));
        break;

        //Node set types
      case XPathResult::UNORDERED_NODE_ITERATOR_TYPE:
      case XPathResult::ORDERED_NODE_ITERATOR_TYPE:
      case XPathResult::UNORDERED_NODE_SNAPSHOT_TYPE:
      case XPathResult::ORDERED_NODE_SNAPSHOT_TYPE:
        resultItem = new DataItemNodeList(navigation->asNodeList(context.get(),false));
        break;

      case XPathResult::ANY_TYPE:
        //Do not do a type change and translate the type for the XPathResult
        resultItem = navigation->asSimpleDataItem(context.get());

        switch (resultItem->getType()) {

          case DataItem::STRINGLITERAL:
            resultType = XPathResult::STRING_TYPE;
            break;

          case DataItem::NUMERICLITERAL:
            resultType = XPathResult::NUMBER_TYPE;
            break;

          case DataItem::NODELIST:
            resultType = XPathResult::UNORDERED_NODE_ITERATOR_TYPE; //default from spec - despite the fact that our nodesets are ordered
            break;

          case DataItem::DATABOOLEAN:
            resultType = XPathResult::BOOLEAN_TYPE;
            break;

          case DataItem::FUNCTION:
          case DataItem::NAVIGATION:
          case DataItem::VARIABLE:
            throw XPathException(XPathException::TYPE_ERR,
                                 "Unknown natural result type requested.");	//XXX: Figure out a better message!
        }//switch

        break;

       default:
          resultItem = new DataItemString;
    }//switch
  }//try
  catch(const DataItemException &)
  {
    throw XPathException(XPathException::TYPE_ERR,
                         "XPathEvaluatorImpl::evaluate(): request and result types different");
  }
  catch(const NavigationException &e)
  {
    throw XPathException(XPathException::TYPE_ERR, e.getError());
  }
  catch(const XPathException &e)
  {
    //rethrow it
    throw XPathException(e.getCode(), e.getString());
  }
  catch (...) {
    throw XPathException(XPathException::TYPE_ERR, "XPathEvaluatorImpl::evaluate(): Unknown exception caught.");
  }

  //Note: Return a new result object, discard passed object
  XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *tmpNode = NULL;
  if(contextNode->getNodeType() == XERCES_CPP_NAMESPACE_QUALIFIER DOMNode::DOCUMENT_NODE) {
    tmpNode = (XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *)contextNode;
  } else {
    tmpNode = contextNode->getOwnerDocument();
  }//if

  return new XPathResultImpl(resultItem, resultType, context.release(), tmpNode);

}//evaluate

