 /*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "XPathResultImpl.hpp"
#include <string>
#include <memory>
#include "../exceptions/NavigationException.hpp"
#include "../exceptions/NodeListException.hpp"

XPathResultImpl::XPathResultImpl(DataItem *res, const XPathResult::resultType resType,
                                 Context *contxt, XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *documentRoot)
   : result(res), resultType(resType), context(contxt), resultNodeList(NULL)
{
	if(resultType == XPathResult::UNORDERED_NODE_ITERATOR_TYPE || resultType == XPathResult::ORDERED_NODE_ITERATOR_TYPE || resultType == XPathResult::UNORDERED_NODE_SNAPSHOT_TYPE || resultType == XPathResult::ORDERED_NODE_SNAPSHOT_TYPE) {
          resultNodeList= result->asNodeList(context, false);
        }

        //Set counter to first node in iteration set
        curIndex = 0;

}//constructor

XPathResultImpl::~XPathResultImpl()
{
   delete result, result=NULL;
   delete resultNodeList, resultNodeList=NULL;
   delete context, context=NULL;
}//destructor

XPathResult::resultType XPathResultImpl::getResultType(void) const
{
	return resultType;
}//getResultType


double XPathResultImpl::getNumberValue(void) const throw (XPathException)
{
	if(resultType != XPathResult::NUMBER_TYPE) {
		throw XPathException(XPathException::TYPE_ERR, errorMessage(XPathResult::NUMBER_TYPE, resultType));
	}//if

	return result->asNum(context);
}//getNumberValue


const XMLCh *XPathResultImpl::getStringValue(void) const throw (XPathException)
{
	if(resultType != XPathResult::STRING_TYPE) {
		throw XPathException(XPathException::TYPE_ERR, errorMessage(XPathResult::STRING_TYPE, resultType));
	}//if

	return result->asString(context);
}//getStringValue

bool XPathResultImpl::getBooleanValue(void) const throw (XPathException)
{
	if(resultType != XPathResult::BOOLEAN_TYPE) {
		throw XPathException(XPathException::TYPE_ERR, errorMessage(XPathResult::BOOLEAN_TYPE, resultType));
	}//if

	return result->asBool(context);
}//getBooleanValue


XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *XPathResultImpl::getSingleNodeValue(void) const throw (XPathException)
{
	if(resultType != XPathResult::ANY_UNORDERED_NODE_TYPE && resultType != XPathResult::FIRST_ORDERED_NODE_TYPE) {
		throw XPathException(XPathException::TYPE_ERR, errorMessage(XPathResult::ANY_UNORDERED_NODE_TYPE, resultType));
	}//if

	auto_ptr<DeadNodeList> resultList(result->asNodeList(context, false));

	if(resultList->getLength() == 0) {
                return NULL;
	} else {
		return resultList->first();
	}//if
}//getSingleNodeValue

bool XPathResultImpl::getInvalidIteratorState(void) const {

        if(resultType != XPathResult::UNORDERED_NODE_ITERATOR_TYPE && resultType != XPathResult::ORDERED_NODE_ITERATOR_TYPE) {
                return false;
        }

        if(resultNodeList->hasDocumentChanged()) {
                return true;
        }

        return false;
}

unsigned long XPathResultImpl::getSnapshotLength(void) const throw(XPathException) {

        if(resultType != XPathResult::UNORDERED_NODE_SNAPSHOT_TYPE && resultType != XPathResult::ORDERED_NODE_SNAPSHOT_TYPE) {
                throw XPathException(XPathException::TYPE_ERR, errorMessage(XPathResult::UNORDERED_NODE_SNAPSHOT_TYPE, resultType));
        }//if

        return resultNodeList->getLength();
}


XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *XPathResultImpl::iterateNext(void) throw (XPathException, XERCES_CPP_NAMESPACE_QUALIFIER DOMException) {

        if(resultType != XPathResult::UNORDERED_NODE_ITERATOR_TYPE && resultType != XPathResult::ORDERED_NODE_ITERATOR_TYPE) {
                throw XPathException(XPathException::TYPE_ERR, errorMessage(XPathResult::UNORDERED_NODE_ITERATOR_TYPE, resultType));
        }//if

        //Check for document changes
	if(resultNodeList->hasDocumentChanged()) {
          throw XERCES_CPP_NAMESPACE_QUALIFIER DOMException(XERCES_CPP_NAMESPACE_QUALIFIER DOMException::INVALID_STATE_ERR, XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("XPathResultImpl::getSetIterator() called and underlying document changed since nodelist creation"));
	}//if

        //Reached end of set, return null
        if(curIndex == resultNodeList->getLength()) {
                return NULL;
        }

        //Otherwise return node at curIndex
        return resultNodeList->item(curIndex++);
}

XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *XPathResultImpl::snapshotItem(unsigned long index) const throw(XPathException) {

        if(resultType != XPathResult::UNORDERED_NODE_SNAPSHOT_TYPE && resultType != XPathResult::ORDERED_NODE_SNAPSHOT_TYPE) {
                throw XPathException(XPathException::TYPE_ERR, errorMessage(XPathResult::UNORDERED_NODE_SNAPSHOT_TYPE, resultType));
        }//if

        //Invalid index, return null
        if(index >= resultNodeList->getLength()) {
          return NULL;
        }

        //Otherwise return node at index
        return resultNodeList->item(index);
}

const std::string XPathResultImpl::errorMessage(XPathResult::resultType requestedType,
                                                XPathResult::resultType resType) const
{
	return typeName(requestedType) + " was requested from a XPathResult of type " + typeName(resType);
}//errorMessage

const std::string XPathResultImpl::typeName(XPathResult::resultType type) const
{
	switch(type) {
        case XPathResult::ANY_TYPE:
                return "ANY_TYPE";

        case XPathResult::NUMBER_TYPE:
                return "NUMBER_TYPE";

        case XPathResult::STRING_TYPE:
                return "STRING_TYPE";

        case XPathResult::BOOLEAN_TYPE:
                return "BOOLEAN_TYPE";

        case XPathResult::ORDERED_NODE_ITERATOR_TYPE:
	case XPathResult::UNORDERED_NODE_ITERATOR_TYPE:
                return "Iterator type";

        case XPathResult::UNORDERED_NODE_SNAPSHOT_TYPE:
        case XPathResult::ORDERED_NODE_SNAPSHOT_TYPE:
                return "Snapshot type";

        case XPathResult::ANY_UNORDERED_NODE_TYPE:
                return "SingleNodeValue";

        case XPathResult::FIRST_ORDERED_NODE_TYPE:
                return "FIRST_ORDERED_NODE_TYPE";

	}
      	return "";
}//typeName
