/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include "VariableStore.hpp"
#include <assert.h>
#include "../context/Context.hpp"
#include "../dataItem/DataItemNumeric.hpp"
#include "../dataItem/DataItemString.hpp"
#include "../dataItem/DataItemNodeList.hpp"
#include "../dataItem/DeadNodeList.hpp"
#include "../DOMutils/v2NSUtils.hpp"
#include "../DOMutils/XMLChStringHash.hpp"
#include "../DOMutils/XMLChBuilder.hpp"

#ifdef TEST
#include "../testharness/TestFunction.hpp"
#endif

VariableStore::VariableStore()
#ifdef TEST
 : TestCase("")
#endif // TEST
{
  /*_scopeStore.reserve(50)*/;
  _scopeStore.push_back(new Scope(Scope::GLOBAL));
}

VariableStore:: ~VariableStore()
{
  // We own all the variable scopes
  for(vector<Scope*>::iterator scopeI = _scopeStore.begin();
      scopeI != _scopeStore.end();
      ++scopeI)
  {
    delete *scopeI;
  }
}

void VariableStore::addScope(Scope *scope)
{
  assert(scope->getType() != Scope::GLOBAL);

  _scopeStore.push_back(scope);
}

void VariableStore::addLocalScope()
{
  addScope(new Scope(Scope::LOCAL));
}

void VariableStore::addLogicalBlockScope()
{
  addScope(new Scope(Scope::LOGICAL_BLOCK));
}

void VariableStore::removeScope()
{
  //assert( _scopeStore[_scopeStore.size() - 1].getType() != GLOBAL );
  delete _scopeStore[_scopeStore.size() - 1];
  _scopeStore.pop_back();
}

void VariableStore::setGlobalVar(const XMLCh *ident,
                                 DataItem *value,
                                 const Context *context)
{
   setGlobalVar(context->getInheritedPrefixNS(v2NSUtils::getPrefix(ident)),
                v2NSUtils::getLocalName(ident), value, context);
}

void VariableStore::setGlobalVar(const XMLCh *namespaceURI,
                                const XMLCh *name,
                                DataItem *value,
                                const Context *context)
{
  XMLCh *expandedName = v2NSUtils::getExpandedName(namespaceURI, name);

  pair<bool, VarHashEntry*> result = _scopeStore[0]->getMap().find(expandedName);
  if(result.first) {
    delete [] expandedName;
    result.second->setValue(value);
  }
  else {
    _scopeStore[0]->getMap().set(expandedName, new VarHashEntry(value));
  }
}

void VariableStore::setVar(const XMLCh *ident,
                           DataItem *value,
                           const Context *context)
{
   setVar(context->getInheritedPrefixNS(v2NSUtils::getPrefix(ident)),
          v2NSUtils::getLocalName(ident), value, context);
}

void VariableStore::setVar(const XMLCh *namespaceURI,
                          const XMLCh *name,
                          DataItem *value,
                          const Context *context)
{
  XMLCh *expandedName = v2NSUtils::getExpandedName(namespaceURI, name);
  int index = find(namespaceURI, name, context);

  pair<bool, VarHashEntry*> result = _scopeStore[index]->getMap().find(expandedName);

  if(result.first) {
    delete [] expandedName;
    result.second->setValue(value);
  }
  else {
    _scopeStore[index]->getMap().set(expandedName, new VarHashEntry(value));
  }
}

void VariableStore::declareVar(const XMLCh *ident,
                               DataItem *value,
                               const Context *context)
{
   declareVar(context->getInheritedPrefixNS(v2NSUtils::getPrefix(ident)),
              v2NSUtils::getLocalName(ident), value, context);
}

void VariableStore::declareVar(const XMLCh *namespaceURI,
                              const XMLCh *name,
                              DataItem *value,
                              const Context *context)
{
  _scopeStore[_scopeStore.size() - 1]->getMap().set(v2NSUtils::getExpandedName(namespaceURI, name),
                                                    new VarHashEntry(value));
}

const DataItem*
VariableStore::getVar(const XMLCh *ident,
                      const Context *context) const
{
  return getVar(context->getInheritedPrefixNS(v2NSUtils::getPrefix(ident)),
                v2NSUtils::getLocalName(ident), context);
}

const DataItem*
VariableStore::getVar(const XMLCh *namespaceURI,
                     const XMLCh *name,
                     const Context *context) const
{
  XMLCh *expandedName = v2NSUtils::getExpandedName(namespaceURI, name);
  pair<bool, VarHashEntry*> result =
    _scopeStore[find(namespaceURI, name, context)]->getMap()
    .find(expandedName);
  delete [] expandedName;
  return result.first ? result.second->getValue() : NULL;
}

const VarHashEntry *VariableStore::getReferenceVar(const XMLCh *ident,
                                                   const Context *context) const
{
  return getReferenceVar(context->getInheritedPrefixNS(v2NSUtils::getPrefix(ident)),
                         v2NSUtils::getLocalName(ident), context);
}

/** Returns a null VarHashEntry if unsuccessful */
const VarHashEntry *VariableStore::getReferenceVar(const XMLCh *namespaceURI,
                                                   const XMLCh *name,
                                                   const Context *context) const
{
  XMLCh *expandedName = v2NSUtils::getExpandedName(namespaceURI, name);
  pair<bool, VarHashEntry*> result =
    _scopeStore[find(namespaceURI, name, context)]->getMap()
    .find(expandedName);
  delete [] expandedName;
  if(result.first) {
    return result.second;
  }
  else {
    return new VarHashEntry(NULL);
  }
}

const DataItem*
VariableStore::getGlobalVar(const XMLCh *ident,
                            const Context *context) const
{
  return getGlobalVar(context->getInheritedPrefixNS(v2NSUtils::getPrefix(ident)),
                      v2NSUtils::getLocalName(ident), context);
}

const DataItem*
VariableStore::getGlobalVar(const XMLCh *namespaceURI,
                           const XMLCh *name,
                           const Context *context) const
{
  XMLCh *expandedName = v2NSUtils::getExpandedName(namespaceURI, name);
  pair<bool, VarHashEntry*> result
     = _scopeStore[0]->getMap().find(expandedName);
  delete [] expandedName;
  return result.first ? result.second->getValue() : NULL;
}

void VariableStore::delVar( const XMLCh *namespaceURI, const XMLCh *name, const Context *context )
{
  XMLCh *expandedName = v2NSUtils::getExpandedName(namespaceURI, name);
  _scopeStore[ find(namespaceURI, name, context) ]->getMap().erase(expandedName);
  delete [] expandedName;
}

void VariableStore::delGlobalVar(const XMLCh *ident,
                                 const Context *context)
{
   delGlobalVar(context->getInheritedPrefixNS(v2NSUtils::getPrefix(ident)),
                v2NSUtils::getLocalName(ident), context);
}

void VariableStore::delGlobalVar( const XMLCh *namespaceURI, const XMLCh *name, const Context *context )
{
  XMLCh *expandedName = v2NSUtils::getExpandedName(namespaceURI, name);
  _scopeStore[0]->getMap().erase(expandedName);
  delete [] expandedName;
}

int VariableStore::find (const XMLCh *namespaceURI, const XMLCh *name, const Context *context) const
{
  // Check the local scopes
  int last = _scopeStore.size() - 1;
  XMLCh *expandedName = v2NSUtils::getExpandedName(namespaceURI, name);
  for(int index = last; index >= 1; index--) {
    if(_scopeStore[index]->getMap().find(expandedName).first) {
	  delete [] expandedName;
      return index;
    }
    if(_scopeStore[index]->getType() == Scope::LOCAL) {
      break;
    }
  }

  // Check the global scope
  if(_scopeStore[0]->getMap().find(expandedName).first) {
    delete [] expandedName;
    return 0;
  }

  // If we can't find it anywhere, then return the top level scope.
  // This lets us set it in the correct place.
  // For gets, we could return any scope.
  delete [] expandedName;
  return last;
}

XMLCh *VariableStore::print() const
{
  XMLChBuilder result;

  vector<Scope*>::const_iterator i = _scopeStore.begin();

  result.appendLiteral("<VariableStore>\n");
  for(; i != _scopeStore.end(); ++i) {
    result.appendLiteral("<Scope type=\"");
    switch((*i)->getType()) {
    case Scope::GLOBAL: {
      result.appendLiteral("global");
      break;
    }
    case Scope::LOCAL: {
      result.appendLiteral("local");
      break;
    }
    case Scope::LOGICAL_BLOCK: {
      result.appendLiteral("logical_block");
      break;
    }
    }
    result.appendLiteral("\">\n");
    result.appendAndDelete((*i)->getMap().print());
    result.appendLiteral("</Scope>\n");
  }
  result.appendLiteral("</VariableStore>\n");

  return result.fetch();
}

#ifdef TEST

#include "../DOMutils/XStr.hpp"

// Test functions

VariableStore::VariableStore(string name) : TestCase(name)
{
  /*_scopeStore.reserve(50)*/;
  _scopeStore.push_back(new Scope(Scope::GLOBAL));
}

void VariableStore::setUp()
{
}

Test* VariableStore::suite()
{
  TestSuite* testSuite = new TestSuite("VarStore");

  testSuite->addTest(new TestCaller<VariableStore> ("testAddScope",
						&VariableStore::testAddScope));
  testSuite->addTest(new TestCaller<VariableStore> ("testRemoveScope",
						&VariableStore::testRemoveScope));
  testSuite->addTest(new TestCaller<VariableStore> ("testSetandGetVars",
						&VariableStore::testSetandGetVars));
  testSuite->addTest(new TestCaller<VariableStore> ("testDelVar",
						&VariableStore::testDelVar));
  return testSuite;
}

void VariableStore::testAddScope()
{
  unsigned int size = _scopeStore.size();
  addScope(new Scope(Scope::LOCAL));
  assert( _scopeStore.size() == (size + 1) );
  assert( _scopeStore[ _scopeStore.size() - 1 ]->getType() == Scope::LOCAL );

  size = _scopeStore.size();
  addScope(new Scope(Scope::LOGICAL_BLOCK));
  assert( _scopeStore.size() == (size + 1) );
  assert( _scopeStore[ _scopeStore.size() - 1 ]->getType() == Scope::LOGICAL_BLOCK );

}

void VariableStore::testRemoveScope()
{
  addScope(new Scope(Scope::LOCAL));
  addScope(new Scope(Scope::LOGICAL_BLOCK));

  unsigned int size = _scopeStore.size();
  removeScope();
  assert( _scopeStore.size() == (size - 1) );

  size = _scopeStore.size();
  removeScope();
  assert( _scopeStore.size() == (size - 1) );
}

void VariableStore::testSetandGetVars()
{
  TestFunction test;
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(test.initDataTree());
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> out(test.initOutputTree());

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *dataRoot = data->getDocumentElement();

  VariableStore store;
  FunctionLookup functionLookup;
  Context c(dataRoot, dataRoot, &store, &functionLookup, XPathResult::ANY_TYPE);
  Context *ec = &c;

  addScope(new Scope(Scope::LOCAL));
  addScope(new Scope(Scope::LOGICAL_BLOCK));
  unsigned int size = _scopeStore[0]->getMap().size();

  setGlobalVar(X(""), X("foobar"), new DataItemNumeric(5.4), ec);
  assert( _scopeStore[0]->getMap().size() == (size + 1) );
  assert( find(X(""), X("foobar"), ec) == 0 );
  const DataItem* answer = getVar(X(""), X("foobar"), ec);

  assert( answer!=NULL );
  assert( answer->asNum( ec ) == 5.4 );

  setVar(X(""), X("charlie"), new DataItemNumeric(3.9), ec);
  answer = getVar(X(""), X("charlie"), ec);
  assert( answer!=NULL );
  assert( answer->asNum( ec ) == 3.9 );

  addScope(new Scope(Scope::LOCAL));

  answer = getVar(X(""), X("charlie"), ec);
  assert( answer==NULL );

  setVar(X(""), X("charlie"), new DataItemNumeric(100), ec);
  answer = getVar(X(""), X("charlie"), ec);
  assert( answer!=NULL );
  assert( answer->asNum( ec ) == 100 );

  setVar(X(""), X("franklin"), new DataItemNumeric(155), ec);
  answer = getVar(X(""), X("franklin"), ec);
  assert( answer!=NULL );
  assert( answer->asNum( ec ) == 155 );

  setVar(X(""), X("tosh"), new DataItemNumeric(2000), ec);
  answer = getVar(X(""), X("tosh"), ec);
  assert( answer!=NULL );
  assert( answer->asNum( ec ) == 2000 );

  answer = getVar(X(""), X("foobar"), ec);

  assert( answer!=NULL );
  assert( answer->asNum( ec ) == 5.4 );

  setVar(X(""), X("foobar"), new DataItemNumeric(1.2), ec);
  answer = getVar(X(""), X("foobar"), ec);

  assert( answer!=NULL );
  assert( answer->asNum( ec ) == 1.2 );

  removeScope();
  removeScope();
  removeScope();

  answer = getVar(X(""), X("foobar"), ec);

  assert( answer!=NULL );
  assert( answer->asNum( ec ) == 1.2 );

  answer = getVar(X(""), X("tosh"), ec);
  assert( answer==NULL );
}

void VariableStore::testDelVar()
{
  addScope(new Scope(Scope::LOCAL));
  addScope(new Scope(Scope::LOGICAL_BLOCK));

  TestFunction test;

  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> data(test.initDataTree());
  auto_ptr<XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument> out(test.initOutputTree());

  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *dataRoot = data->getDocumentElement();

  VariableStore store;
  FunctionLookup functionLookup;
  Context c(dataRoot, dataRoot, &store, &functionLookup, XPathResult::ANY_TYPE);
  Context *context = &c;

  setVar(X(""), X("noggin"), new DataItemNumeric(48), context);
  delVar(X(""), X("noggin"), context);

  const DataItem* answer = getVar(X(""), X("noggin"), context);
  assert( answer==NULL );

  setGlobalVar(X(""), X("finger"), new DataItemNumeric(33), context);
  delVar(X(""), X("finger"), context);

  answer = getVar(X(""), X("finger"), context);
  assert( answer==NULL );

  removeScope();
  removeScope();

  setVar(X(""), X("noggin"), new DataItemNumeric(48), context);
  delVar(X(""), X("noggin"), context);

  answer = getVar(X(""), X("noggin"), context);
  assert( answer==NULL );
}


//*********************
#endif
