/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*

  VariableStore - ementation of the variable store class

*/

#ifndef _VARSTORE_HPP
#define _VARSTORE_HPP

#include <config.h>


#ifdef WIN32
#pragma warning( disable: 4786 )
#pragma warning( disable: 4251 )
#endif

#include <vector>
#include <memory>

#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOM.hpp>

#include "../exceptions/VariableStoreException.hpp"
#include "VarHashEntry.hpp"
#include "Scope.hpp"

#ifdef TEST
#include "../testharness/TestCase.h"
#endif

class DataItem;

using namespace std;

/** implementation of the variable store. Hard codes the global scope as item(0) in the vector. */
class PATHAN_EXPORT VariableStore
#ifdef TEST
  : public TestCase
#endif
{
public:
  /** constructor. automatically creates a global store for the
      global variables. */
  VariableStore();
  /// destructor
  ~VariableStore();


  /** Adds a new scope to the store. Add a LOCAL scope at the
      beginning of *procedural* methods, and a LOGICAL_BLOCK
      scope at the beginning of logical block methods. */
  void addScope(Scope *scope);
  /** Adds a new local scope to the store. */
  void addLocalScope();
  /** Adds a new logical block scope to the store. */
  void addLogicalBlockScope();
  /** Removes the top level scope from the store. To be called at
      the end of methods to implement scoping. */
  void removeScope();


  /** Declares and/or sets a variable in the top level scope. */
  void setVar(const XMLCh *ident,
              DataItem *value,
	      const Context *context);
  void setVar(const XMLCh *namespaceURI,
              const XMLCh *name,
              DataItem *value,
              const Context *context);
  /** Declares and/or sets a variable in the global scope. */
  void setGlobalVar(const XMLCh *ident,
                    DataItem *value,
		    const Context *context);
  void setGlobalVar(const XMLCh *namespaceURI,
                    const XMLCh *name,
                    DataItem *value,
                    const Context *context);


  ///Declares a var in the top level scope
  void declareVar(const XMLCh *ident,
                  DataItem *value,
		  const Context *context);
  void declareVar(const XMLCh *namespaceURI,
                  const XMLCh *name,
                  DataItem *value,
                  const Context *context);


  /** Looks up the value of a variable in the current scope. Returns the
      DataItem value of the variable if successful, else NULL */
  const DataItem* getVar(const XMLCh *ident,
                                     const Context *context) const;
  const DataItem* getVar(const XMLCh *namespaceURI,
                                     const XMLCh *name,
                                     const Context *context) const;
  /** Looks up the value of a variable in the current scope.
      Returns the VarHashEntry for the variable, or null*/
  const VarHashEntry *getReferenceVar(const XMLCh *ident,
                                      const Context *context) const;
  const VarHashEntry *getReferenceVar(const XMLCh *namespaceURI,
                                      const XMLCh *name,
                                      const Context *context) const;
  /** Looks up the value of a variable in the current scope. Returns the
      DataItem value of the variable if successful, else NULL */
  const DataItem* getGlobalVar(const XMLCh *ident,
                                           const Context *context) const;
  const DataItem* getGlobalVar(const XMLCh *namespaceURI,
                                           const XMLCh *name,
                                           const Context *context) const;


  /** Deletes a variable from the current scope. */
  void delVar(const XMLCh *ident,
              const Context *context);
  void delVar(const XMLCh *namespaceURI,
              const XMLCh *name,
              const Context *context);
  /** Deletes a variable from the global scope. */
  void delGlobalVar(const XMLCh *ident,
                    const Context *context);
  void delGlobalVar(const XMLCh *namespaceURI,
                    const XMLCh *name,
                    const Context *context);

  // caller owns
  XMLCh *print() const;

private:
  /** find routine which returns index of scope in the hash table which contains the
      identifiers, or -1 if it can't be found. */
  int find(const XMLCh *namespaceURI,
           const XMLCh *name,
           const Context *context) const;

  vector<Scope*> _scopeStore;

  VariableStore(const VariableStore& rhs);
  VariableStore& operator=(const VariableStore& rhs);

#ifdef TEST
public:
  /// constructor for testing
  VariableStore(string name);
  ///Test setup
  void setUp();
  ///Test body
  static Test *suite();
  ///Test function
  void testAddScope();
  void testRemoveScope();
  void testSetandGetVars();
  void testDelVar();
#endif
};

#endif // _VARSTORE_HPP
