/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */



#include <stdexcept>
#include <cmath>

#include "TestCase.h"
#include "TestResult.h"
#include "estring.h"

// copy constructor
TestCase::TestCase(const TestCase& other)
  : m_name(other.m_name)
{
  // no-op
}


// Create a default TestResult
TestResult *TestCase::defaultResult()
{
  return new TestResult;
}

// Check for a failed general assertion 
void TestCase::assertImplementation (bool          condition,
                                     std::string   conditionExpression,
                                     long          lineNumber,
                                     std::string   fileName) const
{ 
    if (!condition)
        throw CppUnitException (conditionExpression, lineNumber, fileName); 
}


// Check for a failed equality assertion 
void TestCase::assertEquals (long        expected, 
                             long        actual,
                             long        lineNumber,
                             std::string fileName) const
{ 
    if (expected != actual) 
        assertImplementation (false, notEqualsMessage(expected, actual), lineNumber, fileName); 
}


// Check for a failed equality assertion
void TestCase::assertEquals (double        expected, 
                             double        actual, 
                             double        delta,
                             long          lineNumber,
                             std::string   fileName) const
{ 
    if (fabs (expected - actual) > delta) 
        assertImplementation (false, notEqualsMessage(expected, actual), lineNumber, fileName); 

}


// Run the test and catch any exceptions that are triggered by it 
int TestCase::run (TestResult *result)
{
    int errors = 0;
    result->startTest (this);

    setUp ();

    try {
      // Put this line in to output the test names.
      runTest ();
    }
    catch (CppUnitException e) {
        CppUnitException *copy = new CppUnitException (e);
        result->addFailure (this, copy);
	errors = 1;
    }
    catch (std::exception e) {
        result->addError (this, new CppUnitException (e.what ()));
	errors = 1;
    }
    catch (...) {
        CppUnitException *e = new CppUnitException ("unknown exception");
        result->addError (this, e);
	errors = 1;
    }

    tearDown ();

    result->endTest (this);
    return errors;
}


// A default run method
TestResult *TestCase::run ()
{
    TestResult *result = defaultResult ();

    run (result);
    return result;

}


// All the work for runTest is deferred to subclasses 
void TestCase::runTest ()
{
}


// Build a message about a failed equality check 
std::string TestCase::notEqualsMessage (long expected, long actual) const
{ 
    return "expected: " + estring (expected) + " but was: " + estring (actual); 
}


// Build a message about a failed equality check 
std::string TestCase::notEqualsMessage (double expected, double actual) const
{ 
    return "expected: " + estring (expected) + " but was: " + estring (actual); 
}
