/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

/*
 * A command line based tool to run tests.
 * TestRunner prints out a trace as the tests are executed followed by a
 * summary at the end.
 *
 * You can add to the tests that the TestRunner knows about by 
 * making additional calls to "addTest (...)" in main.
 *
 */

#ifdef WIN32
#pragma warning( disable : 4786 )
#endif

#include <iostream>
#include <vector>
#include <locale.h>
#include <xercesc/util/PlatformUtils.hpp>

#include "TextTestResult.h"
#include "Test.h"

// Add declarations for classes that you want to run tests on
#include "TestAllFunctions.h"
#include "TestDatatypes.h"

#include "../DOMutils/v2Utils.hpp"

#include "../simpleVariables/VariableStore.hpp"

using namespace std;

typedef pair<string, Test *> mapping;
typedef vector<pair<string, Test *> > mappings;

//Bloody DOM config stuff.
#define DSL_LOCALE "en_GB"

class TestRunner
{
protected:
  vector<pair<string,Test *> > m_mappings;

public:
  TestRunner    () { _errorCount = 0; }
  ~TestRunner   ();

  void run(void);
  void addTest(string name, Test *test)
  { m_mappings.push_back (mapping (name, test)); }

  int getErrorCount(void) { return _errorCount; };

protected:
  void run(Test *test);
  void printBanner();

private:
  int _errorCount;

};

void TestRunner::printBanner ()
{
  cout << "Usage: driver [ -wait ] testName, "
       << "where name is the name of a test case class" << endl;
}

void TestRunner::run ()
{
  string  testCase;
  int     numberOfTests = 0;
  Test *testToRun = NULL;

  for (mappings::iterator it = m_mappings.begin ();
       it != m_mappings.end ();
       ++it) {
    cout << "Testing : " << (*it).first << endl;
    testToRun = (*it).second;
    run(testToRun);
    numberOfTests++;
  }

  if (numberOfTests == 0) {
    printBanner ();
    return;        
  }
}

TestRunner::~TestRunner ()
{
  for (mappings::iterator it = m_mappings.begin();
       it != m_mappings.end();
       ++it)
    delete it->second;
}

void TestRunner::run (Test *test)
{
  TextTestResult result;

  _errorCount += test->run(&result);

  cout << result << endl;
}

int main (void)
{
  TestRunner runner;

#ifdef UNIX
  char *new_locale = setlocale(LC_CTYPE, DSL_LOCALE);
  if(new_locale == NULL)
    cerr << "Error setting locale: " << DSL_LOCALE;
#endif

  try {
    XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Initialize();
  }
  catch(const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch) {
    cerr << "Exception during Xerces-C initialisation.\n"
         << "Error message:\n"
         << v2Utils::DSLtranscode(toCatch.getMessage()) << endl;
    return 0;
  }

  // add the classes you want to run tests on

  runner.addTest("Datatypes", TestDatatypes::suite());
  runner.addTest("AllFunctions", TestAllFunctions::suite());

  runner.addTest("v2Utils", v2Utils::suite());

  runner.addTest("VarStore", VariableStore::suite());
  
  runner.run();

  return runner.getErrorCount();
}







